VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CD3DPick"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
'  Copyright (C) 1999-2001 Microsoft Corporation.  All Rights Reserved.
'
'  File:       D3DPick.cls
'  Content:    D3D Visual Basic Framework Pick object
'              See raypack and viewport pick entrypoints
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option Explicit

'DOC: Pick record using with CD3DPick class
Public Type D3D_PICK_RECORD
    hit As Long
    triFaceid As Long
    a       As Single
    b       As Single
    dist   As Single
End Type

Dim m_item() As D3D_PICK_RECORD

Dim m_frame() As clsD3DFrame
Dim m_mesh() As clsD3DObject

Dim m_count As Long
Dim m_maxsize As Long
Const kGrowSize = 10
Dim g_identityMatrix As D3DMATRIX

Dim d3dx As D3DX8

'-----------------------------------------------------------------------------
' Name: GetCount
' Dest: returns number of items picked
'-----------------------------------------------------------------------------
Public Function GetCount() As Long
    GetCount = m_count
End Function

'-----------------------------------------------------------------------------
' Name: GetRecord
' Desc: returns the properties of a given pick item
'-----------------------------------------------------------------------------
Public Sub GetRecord(i As Long, ByRef a As Single, ByRef b As Single, ByRef dist As Single, ByRef triFaceid As Long)
    a = m_item(i).a
    b = m_item(i).b
    dist = m_item(i).dist
    triFaceid = m_item(i).triFaceid
End Sub

'-----------------------------------------------------------------------------
' Name: GetFrame
' Desc: returns the frame of a given pick item
'-----------------------------------------------------------------------------
Public Function GetFrame(i As Long) As clsD3DFrame
    Set GetFrame = m_frame(i)
End Function

'-----------------------------------------------------------------------------
' Name: GetMesh
' Desc: returns the mesh of a given pick item
'-----------------------------------------------------------------------------
Public Function GetMesh(i As Long) As clsD3DObject
    Set GetMesh = m_mesh(i)
End Function

'-----------------------------------------------------------------------------
' Name: FindNearest
' Desc: returns the index of the pick with the smallest distance (closest to viewer)
'-----------------------------------------------------------------------------
Public Function FindNearest() As Long
    Dim q As Long, mindist As Single, i As Long
    q = -1
    mindist = 1E+38
    For i = 0 To m_count - 1
        If m_item(i).dist < mindist Then
            q = i
            mindist = m_item(i).dist
        End If
    Next
    FindNearest = q
End Function


'-----------------------------------------------------------------------------
' Name: FindFurthest
' Desc: returns the index of the pick with the largest distance
'-----------------------------------------------------------------------------
Public Function FindFurthest() As Long
    Dim q As Long, maxdist As Single, i As Long
    q = -1
    maxdist = -1E+38
    For i = 0 To m_count - 1
        If m_item(i).dist < maxdist Then
            q = i
            maxdist = m_item(i).dist
        End If
    Next
    FindFurthest = q
End Function


'-----------------------------------------------------------------------------
' Name: Destroy
' Desc: Release all references
'-----------------------------------------------------------------------------
Public Function Destroy()
    ReDim m_mesh(0)
    ReDim m_frame(0)
    m_count = 0
    m_maxsize = 0
End Function


'-----------------------------------------------------------------------------
' Name: ViewportPick
' Params:
'    frame      parent of frame heirarchy to pick from
'    x          x screen coordinate in pixels
'    y          y screen coordinate in pixels
'
' Note: After call GetCount to see if any objets where hit
'-----------------------------------------------------------------------------
'Public Function ViewportPick(frame As CD3DFrame, x As Single, y As Single)
Public Function ViewportPick(frame As clsD3DFrame, X As Single, Y As Single, d3ddevice As Direct3DDevice8, world As D3DMATRIX)
    
    Destroy
    
    Dim viewport As D3DVIEWPORT8
    'Dim world As D3DMATRIX
    Dim proj As D3DMATRIX
    
    Dim v As D3DVECTOR
    
    d3ddevice.GetTransform D3DTS_PROJECTION, proj
    v.X = ((2 * X) - 1) / proj.m11
    v.Y = ((2 * Y) - 1) / proj.m22
    v.z = 1
    
    ' Get the Inverse View Matrix
    Dim view As D3DMATRIX, m As D3DMATRIX
    d3ddevice.GetTransform D3DTS_VIEW, view
    D3DXMatrixInverse m, 0, view
    
    Dim vPickRayDir As D3DVECTOR
    Dim vPickRayOrig As D3DVECTOR
    
    ' Transform the screen space pick ray into 3d space
    vPickRayDir.X = v.X * m.m11 + v.Y * m.m21 + v.z * m.m31
    vPickRayDir.Y = v.X * m.m12 + v.Y * m.m22 + v.z * m.m32
    vPickRayDir.z = v.X * m.m13 + v.Y * m.m23 + v.z * m.m33
    vPickRayOrig.X = m.m41
    vPickRayOrig.Y = m.m42
    vPickRayOrig.z = m.m43
    
    Dim bHit As Boolean
    
    Dim item As D3D_PICK_RECORD
    
    'Check all child meshes
    'Even if we got a hit we continue as the next mesh may be closer
    Dim childMesh As clsD3DObject
    Dim i As Integer
    
    For i = 1 To frame.GetChildMeshCount
        
        Set childMesh = frame.GetChildMesh(i)
        
        If Not childMesh.mesh Is Nothing Then
            d3dx.Intersect childMesh.mesh, vPickRayOrig, vPickRayDir, item.hit, item.triFaceid, item.a, item.b, item.dist, 0
        End If
        
        
        Debug.Print X, Y, item.hit
        If item.hit <> 0 Then
            InternalAddItem frame, childMesh, item
            item.hit = 0
        End If
        bHit = True
    Next
    
    'check pick for all child frame
    Dim childFrame As clsD3DFrame
    For i = 1 To frame.GetChildFrameCount
        Set childFrame = frame.GetChildFrame(i)
        bHit = bHit Or ViewportPick(childFrame, X, Y, d3ddevice, world)
    Next

    ViewportPick = bHit
    
    'NOTE the following functions will fail on PURE HAL devices
    'use ViewportPickEx if working with pureHal devices
    
    'd3ddevice.GetViewport viewport
    'world = g_identityMatrix
    'd3ddevice.GetTransform D3DTS_VIEW, view
    
    'ViewportPick = ViewportPickEx(frame, viewport, proj, view, world, X, Y)
    
End Function

'-----------------------------------------------------------------------------
' Name: ViewportPickEx
' Desc: Aux function for ViewportPick
'-----------------------------------------------------------------------------
Public Function ViewportPickEx(frame As clsD3DFrame, viewport As D3DVIEWPORT8, proj As D3DMATRIX, view As D3DMATRIX, world As D3DMATRIX, X As Single, Y As Single) As Boolean
    
    'If frame.Enabled = False Then Exit Function
    
    Dim d3dx As D3DX8
    Set d3dx = New D3DX8
    
    Dim vIn As D3DVECTOR, vNear As D3DVECTOR, vFar As D3DVECTOR, vDir As D3DVECTOR
    Dim bHit As Boolean, i As Integer
    
    If frame Is Nothing Then Exit Function
        
    Dim currentMatrix As D3DMATRIX
    Dim NewWorldMatrix As D3DMATRIX
    
    currentMatrix = frame.GetMatrix
        
    'Setup our basis matrix for this frame
    D3DXMatrixMultiply NewWorldMatrix, currentMatrix, world
    
    vIn.X = X:    vIn.Y = Y
    
    'Compute point on Near Clip plane at cursor
    vIn.z = 0
    D3DXVec3Unproject vNear, vIn, viewport, proj, view, NewWorldMatrix
    
    'compute point on far clip plane at cursor
    vIn.z = 1
    D3DXVec3Unproject vFar, vIn, viewport, proj, view, NewWorldMatrix

    'Debug.Print vNear.X & " / " & vNear.Y & " / " & vNear.z
    'Debug.Print vFar.X & " / " & vFar.Y & " / " & vFar.z

    'Comput direction vector
    D3DXVec3Subtract vDir, vFar, vNear
    'Debug.Print vDir.X & " / " & vDir.Y & " / " & vDir.z
                
    Dim item As D3D_PICK_RECORD
    
    'Check all child meshes
    'Even if we got a hit we continue as the next mesh may be closer
    Dim childMesh As clsD3DObject
    For i = 1 To frame.GetChildMeshCount
        
        Set childMesh = frame.GetChildMesh(i)
        
        If Not childMesh.mesh Is Nothing Then
            d3dx.Intersect childMesh.mesh, vNear, vDir, item.hit, item.triFaceid, item.a, item.b, item.dist, 0
        End If
        
        If item.hit <> 0 Then
            InternalAddItem frame, childMesh, item
            item.hit = 0
        End If
        bHit = True
    Next
    
    'check pick for all child frame
    Dim childFrame As clsD3DFrame
    For i = 1 To frame.GetChildFrameCount
        Set childFrame = frame.GetChildFrame(i)
        bHit = bHit Or ViewportPickEx(childFrame, viewport, proj, view, NewWorldMatrix, X, Y)
    Next

    ViewportPickEx = bHit

End Function

'-----------------------------------------------------------------------------
' Name: RayPick
' Desc: given a ray cast it into a scene graph
' Params:
'    frame      parent of frame heirarchy to pick from
'    vOrig      origen of the ray to cast
'    vDir       direction of the ray
'
' Note: the following functions will fail on PURE HAL devices
'       use RayPickEx if working with pureHal devices
'       Call getCount to see if the ray hit any objects
'
'-----------------------------------------------------------------------------
'Public Function RayPick(frame As CD3DFrame, vOrig As D3DVECTOR, vDir As D3DVECTOR)
'    Destroy
'
'    Dim world As D3DMATRIX
'
'
'    g_dev.GetTransform D3DTS_WORLD, world
'
'    RayPick = RayPickEx(frame, world, vOrig, vDir)
'
'End Function

'-----------------------------------------------------------------------------
' Name: RayPickEx
' Desc: Aux function for RayPickEx
'-----------------------------------------------------------------------------
'Public Function RayPickEx(frame As CD3DFrame, worldmatrix As D3DMATRIX, vOrig As D3DVECTOR, vDir As D3DVECTOR) As Boolean
'
'    Dim NewWorldMatrix As D3DMATRIX     'world matrix for this stack frame
'    Dim InvWorldMatrix As D3DMATRIX     'world matrix for this stack frame
'    Dim currentMatrix As D3DMATRIX
'    Dim i As Long, det As Single, bHit As Boolean
'    Dim vNewDir As D3DVECTOR, vNewOrig As D3DVECTOR
'
'    If frame Is Nothing Then Exit Function
'
'    currentMatrix = frame.GetMatrix
'
'    'Setup our basis matrix for this frame
'    D3DXMatrixMultiply NewWorldMatrix, currentMatrix, worldmatrix
'    D3DXMatrixInverse InvWorldMatrix, det, NewWorldMatrix
'
'
'    ' we want to compute vdir and vOrig in model space
'    ' note we use TransformNormal so we dont translate vDir
'    ' just rotate it into a new dir
'    Call D3DXVec3TransformNormal(vNewDir, vDir, InvWorldMatrix)
'    Call D3DXVec3TransformCoord(vNewOrig, vOrig, InvWorldMatrix)
'
'
'    Dim item As D3D_PICK_RECORD
'
'
'    'Check all child meshes
'    'Even if we got a hit we continue as the next mesh may be closer
'    Dim childMesh As CD3DMesh
'    For i = 0 To frame.GetChildMeshCount() - 1
'
'        Set childMesh = frame.GetChildMesh(i)
'
'        If Not childMesh Is Nothing Then
'            Call D3DXVec3Scale(vDir, vDir, 1000) 'Workaround for d3dx Intersect bug
'            g_d3dx.Intersect childMesh.mesh, vNewOrig, vDir, item.hit, item.triFaceid, item.a, item.b, item.dist, 0
'        End If
'
'        If item.hit <> 0 Then
'            InternalAddItem frame, childMesh, item
'            item.hit = 0
'        End If
'        bHit = True
'    Next
'
'    'check pick for all child frame
'    Dim childFrame As CD3DFrame
'    For i = 0 To frame.GetChildFrameCount() - 1
'        Set childFrame = frame.GetChildFrame(i)
'        bHit = bHit Or _
'            RayPickEx(childFrame, NewWorldMatrix, vOrig, vDir)
'    Next
'
'    RayPickEx = bHit
'
'End Function


'-----------------------------------------------------------------------------
' InternalAddItem
'-----------------------------------------------------------------------------
Private Sub InternalAddItem(parentFrame As clsD3DFrame, mesh As clsD3DObject, item As D3D_PICK_RECORD)
    Dim maxsize As Long
    If m_maxsize = 0 Then
        ReDim m_item(kGrowSize)
        ReDim m_mesh(kGrowSize)
        ReDim m_frame(kGrowSize)
        m_maxsize = kGrowSize
    ElseIf m_count >= m_maxsize Then
        ReDim Preserve m_item(m_maxsize + kGrowSize)
        ReDim Preserve m_frame(m_maxsize + kGrowSize)
        ReDim Preserve m_mesh(m_maxsize + kGrowSize)
        m_maxsize = m_maxsize + kGrowSize
    End If
    
    Set m_mesh(m_count) = mesh
    Set m_frame(m_count) = parentFrame
    m_item(m_count) = item
                             
    m_count = m_count + 1
End Sub

Private Sub Class_Initialize()
  Set d3dx = New D3DX8
End Sub

Private Sub Class_Terminate()
  Set d3dx = Nothing
End Sub
