Attribute VB_Name = "modHW2shipHOD"
Option Explicit

'********************************************************************************

Public Type texCo
    u As Single
    V As Single
End Type

Public Type triangle
    vtx1 As Integer
    vtx2 As Integer
    vtx3 As Integer
End Type

Public Type doubleV3
    X As Double
    Y As Double
    Z As Double
End Type

Public Type fHdr
    tF As String * 4    ' form type (e.g. 'FORM' or 'NRML')
    lF As Long          '   # bytes covering this form (Reversed Long)
End Type

'********************************************************************************

Public Type ownrB
    tID As String * 4   ' type ID = 'OWNR'
    lF As String * 4    '   # bytes covering 'OWNR' form (are there other 'INFO' sub-forms)? -> yes!
    lN As Long          '   # bytes in owner name
    nOw As String       ' owner name
End Type

Public Type infoB
    tID As String * 4   ' type ID = 'INFO'
    ownrFb As ownrB
End Type

Public Type ssubB       ' SSUB contains a listing of face-indices (starting at zero) of faces as outlined in the corresponding TRIS chunk.
                        '  But why would Relic want to list faces? And is there a certain grouping?
    tID As String * 4   ' type ID = 'SSUB'
    altLf As Long       ' *** Used alternatively to contain form length (# bytes) if NO 'fHdr' is used.
                        '    NB: contains '1000' or {00 00 03 E8} in BSRM form.
'***
    flag(1 To 4) As Long   ' ?
    fcIdx() As Integer    ' Listing of face-indices (starting at zero) of faces as outlined in the corresponding TRIS chunk.
End Type

Public Type vnrmB       ' Combinations of vertex normals to make up the face normals for the faces in the corresponding TRIS chunk.
    tID As String * 4   ' type ID = 'VNRM'
    altLf As Long       ' *** Used alternatively to contain form length (# bytes) if NO 'fHdr' is used.
                        '    NB: contains '1000' or {00 00 03 E8} in BSRM form. (Reversed Long).
    noNrmls As Long     ' number of normals
    nrml() As D3DVECTOR ' Interpreted as vertex normals (normalized, three per face), for calculating the resulting face normal.
    noFcs As Long       ' (number of faces) * 3   -> Should comply w. number of TRIS-faces.
    fc() As triangle    ' Not really a triangle, moreover the indices of three normals to be combined
                        '  into one face normal per TRIS-face.
End Type

Public Type trisB       ' Plain triangles
    tID As String * 4   ' type ID = 'TRIS'
    lF As Long          '   # bytes to follow in this chunk (seen when this chunk is part of a 'COLD' form).
                        '    NB: contains '1000' or {00 00 03 E8} in BSRM form. (Reversed Long).
    noVtcs As Long      ' number of vertices
    vtx() As D3DVECTOR  ' this time no vertex normals and texture coordinates attached (unlike BMSH vertices).
    noFcs As Long       ' (number of faces) * 3  (*** No separator used).
    fc() As triangle
End Type

Public Type bsphB       ' Interpreted as collision sphere.
    tID As String * 4   ' type ID = 'BSPH'
    lF As Long          '   # bytes to follow in this chunk (seen when this chunk is part of a 'COLD' form).
                        '    NB: contains '1000' or {00 00 03 E8} in BSRM form. (Reversed Long).
    centre As D3DVECTOR ' checked OK in drawing program.
    radius As Single    ' checked OK in drawing program.
End Type

Public Type bboxB       ' Interpreted as collision box.
    tID As String * 4   ' type ID = 'BBOX'
    lF As Long          '   # bytes to follow in this chunk (seen when this chunk is part of a 'COLD' form).
                        '    NB: contains '1000' or {00 00 03 E8} in BSRM form. (Reversed Long).
    leftLowerCorner As D3DVECTOR ' checked OK in drawing program.
    edgeSizes As D3DVECTOR       ' checked OK in drawing program.
End Type

Public Type descB
    tID As String * 4   ' type ID = 'DESC'
    altLf As Long       ' *** Used alternatively to contain form length (# bytes) if NO 'fHdr' is used.
                        '    NB: contains '1000' or {00 00 03 E8} in BSRM form. (Reversed Long).
    lN As Long          '   # bytes in mesh name
    nM As String        ' mesh name
    lO As Long          '   # bytes in object name
    nO As String        ' object name
    nonGoblin As Long   ' Goblin indicator.
End Type

'***************************************************************************************************
'   IMPORTANT RULE: IT SEEMS THAT THE NUMBER OF BMSH CHUNKS IS DEFINED BY:
'
'       nr. MUTLs + nr. GOBGs = nr. BMSHs
'
'   -> CHECK BUILT IN !
'***************************************************************************************************

Public Type bsrmB
    tID As String * 4   ' type ID = 'BSRM'
    descFh As fHdr      ' with 'NRML'
    descFb As descB     ' occurs only once in BSRM chunk
    bboxFh As fHdr      ' with 'NRML'
    bboxFb As bboxB     ' occurs only once in BSRM chunk
    bsphFh As fHdr      ' with 'NMRL'
    bsphFb As bsphB     ' occurs only once in BSRM chunk
    trisFh As fHdr      ' with 'NMRL'
    trisFb As trisB     ' occurs only once in BSRM chunk
    vnrmFh As fHdr      ' with 'NMRL'
    vnrmFb As vnrmB     ' occurs only once in BSRM chunk
    ssubFh As fHdr      ' with 'NMRL'
    ssubFb As ssubB     ' occurs only once in BSRM chunk
End Type

Public Type coldB
    tID As String * 4   ' type ID = 'COLD'
    lO As Long          '   # bytes in object name to follow.
    nO As String        ' Hierarchy object name.
    bboxFb As bboxB     ' occurs once in COLD chunk, without header.
    bsphFb As bsphB     ' occurs once in COLD chunk, without header.
    trisFb As trisB     ' occurs once in COLD chunk, without header.
End Type

Public Type bndvB       ' Interpreted as 'BouNDary Values' -> LARGER than mesh extremes
                        ' Could be something like: 'camara can't zoom closer than'.
    tID As String * 4   ' type ID = 'BNDV'
    lF As Long          '   # bytes in form
    lN As Long          '   # bytes in name to follow (usually 5).
    nAll As String      ' = 'Whole'
    lO As Long          '   # bytes in object name
    nO As String        ' object name
    noGenPts As Long              ' = always 8 (like: generate EIGHT corners from points below).
    leftLowerCorner As D3DVECTOR  ' checked OK in drawing program.
    sepr1(1 To 15) As Long        ' = always all zero
    rightUpperCorner As D3DVECTOR ' checked OK in drawing program.
    sepr2(1 To 4) As Long         ' = always all zero
End Type

'***************************************************************************************************
'   --- start ANIM chunk def.

Public Type keyfD
    kTime As Double     ' Time values ( x 1/30th again! ).
'*** Check whether these are really like their variable name!
    kPos As D3DVECTOR
    kRot As D3DVECTOR
End Type

Public Type animB
    tID As String * 4
    noAnimPars As Long  ' number of animation parameters in this form = always 1 ? (Reversed Long).
    lNpar As Long       '   # bytes in animation parameter name.
    nPar As String      ' animation parameter name.
    noKeyfs As Long     ' number of keyframes
    keyframe() As keyfD
    sepr(1 To 2) As Long    ' or one double? = always zero.
End Type

'   --- end ANIM chunk def.
'   --- start MRKR chunk def.

Public Type keyfB
    tID As String * 4   ' type ID = 'KEYF'
    ' No more data in KEYF form seen anywhere. Regarded as trailer for the MRKR form.
End Type

Public Type headB
    tID As String * 4   ' type ID = 'HEAD'
    noHds As Long       ' number of markers in this form (= always 1, Reversed Long).
    lN As Long          '   # bytes in name
    nHe As String       ' marker name
    lNpo As Long        '   # bytes in type name for marker connected object or joint.
    nPo As String       ' Mostly 'Root', also seen 'moshipdoor', so may point to parent hierarchy object.
                        '  With the vgr_planetkiller, all 'Joint$#' are named here.
'***
    sepr As Double          ' = always 0 ?  No, see vgr_infiltratorpod.hod, 'markerB'.
    position As doubleV3    ' found after development of the kpr_sajuuk.hod markers.
    rotation As doubleV3    ' found after development of the kpr_sajuuk.hod markers.
    keyfFh As fHdr          ' with 'FORM'
    keyfFb As keyfB         ' occurs only once per HEAD chunk.
End Type

Public Type mrkrB
    tID As String * 4   ' type ID = 'MRKR'
    headFh As fHdr      ' with 'NRML'
    headFb As headB     ' occurs only once per MRKR chunk.
End Type

'   --- end MRKR chunk def.

'***************************************************************************************************
' Hierarchy objects start

'   --- start Dock paths def.

Public Type pathPoint
    ' 64 bytes per point
    pCo As D3DVECTOR    'position?
    pOr As D3DVECTOR    'orientation?
'*** Below: probably spline parameters or flags
    flag0 As Long       ' must be Long!
    flag1 As Single     ' must be Single!
    flag2 As Long       ' must be Long!
    flag3 As Single     ' must be Single!
    flag4() As Long     ' must be Long!
End Type

Public Type shpPath
    lN As Long          '   # bytes in path name
    nP As String        ' path name
    lO As Long          '   # bytes in hierarchy object name
    nO As String        ' name of the hierarchy object this path is connected to.
                        '  (usually 'world', i.e. path described in coordinates relative to
                        '    whole ship).
    unknown8 As Long    ' = always  6
    noPars As Long      ' Says how many parameters / flags are in the path-points.
    flag1 As Long       ' Flags are either 0 or 1 (Delphy).
    flag2 As Long       '  They're not excluding each other (see paths of hgn_moship).
    flag3 As Long
    lNSfs As Long       '   # bytes in names of ship families to use this path.
    nSfs As String      ' names of ship families to use this path, separated by comma & space.
    flag4 As Long       ' This parameter is either 1 or 0 (Pesmontis).
    lAPs As Long        '   # bytes in names of alternative (?) paths.
    nAPs As String      ' alternative path names, separated by comma & space.
    noPpoints As Long   ' number of points in this path.
    pP() As pathPoint
End Type

Public Type dockB       ' Docking paths don't seem to be necessary for capture points.
    tID As String * 4   ' type ID = 'DOCK'
    lF As Long          '   # bytes in DOCK chunk
    noPaths As Long     ' number of dock paths in DOCK chunk.
    path() As shpPath
End Type

'   --- end Dock paths def.
'   --- start Navlight def.

Public Type shpNavl     ' Data chunk per navlight
    lHo As Long         '   # bytes in navlight name
    nHo As String       ' name of hierarchy object to connect to (i.e. navlight name, because these appear in the hierarchy too).
'***
    n1 As Single        ' RGBA / ARGB values? -> Delphy
    n2 As Single        ' all Single -> Delphy
    n3 As Single
    n4 As Single
    lT As Long          '   # bytes in navlight type-name
    tID As String       ' navlight type name
'***
    n5 As Single        ' all Single -> Delphy
    n6 As Single
    n7 As Single
    n8 As Single
    n9 As Single        ' Not always equal to 'n2': see NavLight_01, NavLight_02 & NavLight_03 of hgn_moship.
    sepr(1 To 2) As Byte    ' Each of these bytes can either be {00} or {01}.
End Type

Public Type navlB
    tID As String * 4   ' type ID = 'NAVL'
    altLf As Long       '  (= always 3) (Reversed Long).
                        ' *** Used alternatively to contain form length (# bytes) if NO 'fHdr' is used.
    noNavls As Long     ' number of navlights in this form
    navl() As shpNavl
End Type
    
'   --- end Navlight def.
    
Public Type burnB
    tID As String * 4   ' type ID = 'BURN'
    lF As Long          '   # bytes in BURN chunk (Reversed Long).
    lN As Long          '   # bytes in burn name
    nB As String        ' burn name
    lHo As Long         '   # bytes in hierarchy object name
    nHo As String       ' name of hierarchy object to connect to.
    noConnPnts As Long    ' (= always 5) may be number of connection points.
    unknown4 As Long    ' (= always 1)
    cPnt() As D3DVECTOR ' 'Points-in-a-row': like connection points at different LODs (because of NLIPS trouble?).
End Type

'    --- Start ETSH def.

Public Type etshD
    noConnPnts As Long  ' Number of connection points (can be variable for various ETSH chunks in same HOD).
    cPnt() As D3DVECTOR
End Type

Public Type etshB       ' "Engine Trail SHape" chunk
    tID As String * 4   ' type ID = 'ETSH'
    lF As Long          '   # bytes in ETSH chunk (Reversed Long).
    lN As Long          '   # bytes in shape name
    nS As String        ' Shape name
    lHo As Long         '   # bytes in hierarchy object name
    nHo As String       ' Name of hierarchy object to connect to.
    tSet() As etshD     ' The hgn_pulsarcorvette shows multiple parts within one ETSH chunk.
End Type

'   --- End ETSH def.

'***************************************************************************************************
' BMSH chunk definition (!) - INTERMEZZO -
'  (placed here because used by GLOW, GOBG and MULT chunks).

Public Type vtx1B       ' "1B" relates to the 'MULT' BMSH identifier.
    vCo As D3DVECTOR    ' vertex coordinates
    sepr1 As Long       ' '{00 00 80 3F} (interpreted as a separator)
    VN As D3DVECTOR     ' vertex normal
    sepr2 As Long       ' '{00 00 80 3F} (interpreted as a separator)
    tCo As texCo        ' texture coordinates
End Type

Public Type shpMesh
    texTag As Long      ' pointer to a shader (therefor points to a texture group)
    dwordsPerVtx As Long    '  =  {1B 00 00 00} for ship-mesh BMSH chunks,
                            ' but {0B 00 00 00} for ship-glow BMSH chunks,
                            ' and {05 00 00 00} for background BMSH chunks
                            ' => to be used as additional identifier?
    noVtcs As Long      ' number of vertices in this mesh part.
    shipVtx() As vtx1B
    sepr(1 To 6) As Byte    ' '{01 00 02 02 00 00}  (understood as separator)
                            '  Also seen "17306200" {AD 00 06 02 00 00}, in hgn_assaultcorvetteelite.hod ->
                            '    -> that mesh's faces can't be read (yet).
    noFcs As Long       ' (number of faces) * 3 in this mesh part.
    shipFc() As triangle
End Type

Public Type bmshB
    tID As String * 4   ' type ID = 'BMSH'
    altLf As Long       ' *** Used alternatively to contain form length (# bytes) if NO 'fHdr' is used.
    LOD As Long         ' LOD number. Contains zero when used in GLOW chunks.
    noMshs As Long      ' number of meshes in this BMSH chunk.
    shipmesh() As shpMesh
End Type

Public Type bmshC
    bmshFh As fHdr
    bmshFb As bmshB
End Type

'***************************************************************************************************
' Hierarchy objects (cont'd).

Public Type glowB       ' Appears directly after ETSH chunk (see sp_tanker.hod).
    tID As String * 4   ' type ID = 'GLOW'
    tFi As String * 4   ' with 'NRML'
    lFi As Long         '  # bytes in INFO chunk.
    tIDi As String * 4  ' type ID = 'INFO'. NB: special type of INFO chunk, not separately defined.
    altLfi As Long      ' *** Used alternatively to contain form length (# bytes) if NO 'fHdr' is used.
    lN As Long          '   # bytes in mesh name.
    nGl As String       ' glow-mesh name.
    lHo As Long         '   # bytes in hierarchy object name.
    nHo As String       ' name of hierarchy object to connect to.
    LOD As Long         ' LOD number. Now a BMSH chunk LOD number contains only zero.
    bmshFh As fHdr      ' occurs only ONCE per GLOW chunk.
    bmshFb As bmshB     ' occurs only ONCE per GLOW chunk.
End Type

Public Type hierObj
    lO As Long              '   # bytes in object name.
    nO As String            ' object name
    lP As Long              '   # bytes in parent name ( = 0 for 'Root' object).
    nP As String            ' parent name
    cldPos As D3DVECTOR     ' position relative to parent.
    cldRot As D3DVECTOR     ' orientation relative to parent.
    cldScl As D3DVECTOR     ' possibly scale, but not sure (seen (1,1,1) only).
    cldWrl As D3DVECTOR     ' position relative to root object (according to development of vgr_planetkiller.hod).
    sepr(1 To 3) As Byte    ' {01 01 01} understood as separator.
End Type

Public Type hierB
    tID As String * 4   ' with 'HIER'
    lF As Long          '   # bytes in HIER chunk
    noHobjects As Long  ' number of objects in hierarchy.
    hObj() As hierObj
End Type

'***************************************************************************************************

Public Type glowC
    glowFh As fHdr      ' with 'FORM'
    glowFb As glowB
End Type

Public Type animC       ' with 'FORM'
    animFh As fHdr
    animFb As animB
End Type

Public Type mrkrC
    mrkrFh As fHdr      ' with 'FORM'
    mrkrFb As mrkrB
End Type

Public Type coldC       ' with 'FORM'
    coldFh As fHdr
    coldFb As coldB
End Type

Public Type bsrmC
    bsrmFh As fHdr      ' with 'FORM'
    bsrmFb As bsrmB
End Type

Public Type dtrmB       ' The order in which some of these chunks appear probably follows their naming
                        '  sequence in the HIER chunk.
    tID As String * 4   ' type ID = 'DTRM'
    hierFb As hierB     ' ship type HODs, including asteroids & similar, contain at least one HIER chunk.
                        '   NB: no HIER-header used.
    etshFb() As etshB   ' ETSH chunks can occur multiple times, and do NOT exclude GLOW chunks (see sp_tanker.hod).
                        '   NB: no ETSH header used. Within an ETSH chunk there can be more than one set op points...
    glowCh() As glowC   ' GLOW chunks can occur multiple times, and exclude BURN chunks.
    burnFb() As burnB   ' BURN chunks can occur multiple times, and exclude GLOW chunks.
                        ' (ETSH does NOT always include BURN -> hgn_shipyard, hgn_ecmprobe).
                        '  (# o. BURN chunks does not always match # o. engines in HIER chunk: hgn_interceptor).
                        '   NB: no BURN-header used.
    navlFh As fHdr      ' form type = 'NRML'. NB: NOT a 'FORM' type but an 'NRML' type (!).
                        '  This probably HAS to be, read-algorith-wise!
    navlFb As navlB     ' a NAVL chunk appears only ONCE if at all.
    dockFb As dockB     ' a DOCK chunk appears only ONCE if at all. NB: no DOCK-header used.
    animCh() As animC   ' doesn't occur in EVERY ship type HOD.
    mrkrCh() As mrkrC   ' doesn't occur in EVERY ship type HOD.
    bndvFb As bndvB     ' occurs at least ONCE as if closing off the HIER chunk. No header used.
    coldCh() As coldC   ' can occur multiple times (e.g. hgn_gunturret.hod), but can also never appear.
    bsrmCh() As bsrmC   ' every ship type HOD contains at least one BSRM chunk.
End Type

'***************************************************************************************************

Public Type gobgB       ' "GOBlin Group" chunk.
    tID As String * 4   ' type ID = 'GOBG'
    altLf As Long       ' *** Used alternatively to contain form length (# bytes) if NO 'fHdr' is used.
    lN As Long          '   # bytes in goblin group name.
    nG As String        ' goblin name
    lO As Long          '   # bytes in hierarchy object name.
    nO As String        ' hierarchy object name.
    bmshFh As fHdr      ' only ONE BMSH chunk per GOBG chunk.
    bmshFb As bmshB     ' only ONE BMSH chunk per GOBG chunk.
End Type

Public Type multB       ' "MULTi-mesh" chunk
    tID As String * 4   ' type ID = 'MULT'
    altLf As Long       ' *** Used alternatively to contain form length (# bytes) if NO 'fHdr' is used.
    lN As Long          '   # bytes in multi-mesh group name.
    nM As String        ' multi-mesh name
    lO As Long          '   # bytes in hierarchy object name.
    nO As String        ' hierarchy object name (for hierarchy).
    noMeshes As Long    ' number of meshes in this MULT chunk.
    bmshCh() As bmshC
End Type

'***************************************************************************************************

Public Type dxt5B
    tID As String * 4   ' type ID = 'DXT5' or '8888' (for TGA)
    noMIPls As Long     ' number of MIP MAP levels
    Wi As Long          ' image width
    Hi As Long          ' image height
    Body() As Byte      ' image contents: can be DDS OR (!) TGA !
End Type

Public Type lmipB
    tID As String * 4   ' type ID = 'LMIP'
    lF As Long          '   # bytes in complete LMIP chunk, incl. DXT5 chunk.
    lFDN As Long        '   # bytes in {drive, directory & texture file name}.
    nFDN As String      ' drive, directory & texture file name.
    dxt5Fb As dxt5B     ' occurs only ONCE per LMIP chunk.
End Type

'***************************************************************************************************
'   --- start Shader def.

Public Type shpShader
    unknown1 As Long    ' '5 or {05 00 00 00}
    unknown2 As Long    ' '4 or {04 00 00 00}
    texTag  As Long     ' Tag used in the BMSH chunk for texture assignment.
                        '  This number can become -1, e.g. with the hgn_assaultcorvetteelite.hod and the hgn_attackbomberelite.hod.
    lShadN As Long      '   # bytes for the shader name.
                        '  This number can be {FF FF FF FF} (!), e.g. with the hgn_assaultcorvetteelite.hod and the hgn_attackbomberelite.hod.
                        '   With the cap_chunk_01.hod, it's possible to run into 'STAT', i.e. no name-length but a string!
    nShad As String     ' shader name.
End Type

Public Type statB
    tID As String * 4   ' type ID = 'STAT'
    altLf As Long       ' *** Used alternatively to contain form length (# bytes) if NO 'fHdr' is used.
    lTgrpN As Long      '   # bytes for the name of the texture group.
    nTgrp As String     ' name of the texture group
    lType As Long       '   # bytes for the type name of the texture group
    nType As String     ' shading types a.o. 'ship' (typically contains alpha for team/stripe),
                        '  'default' (with 'lambert1'-STAT), 'thruster' (typically w. '$diffuseOn' and '$diffuseOff'),
                        '   'badge' (a.o. w. shader '$mask'), 'matte', 'matteAlpha', 'diffuseA' & 'diffuseB' (-> bump map for asteroid!), ...
    noShaders As Long   ' Number of shaders in this texture group.
    shader() As shpShader
End Type

'   --- end Shader def.
'***************************************************************************************************

Public Type statC
    statFh As fHdr    ' with 'NRML'
    statFb As statB
End Type

Public Type multC
    multFh As fHdr    ' with 'NRML'
    multFb As multB
End Type

Public Type gobgC
    gobgFh As fHdr    ' with 'NRML'
    gobgFb As gobgB   ' NOT ALL ship type HODs contain goblins!!
End Type

Public Type hvmdB
    tID As String * 4   ' type ID = 'HVMD'
    statCh() As statC
    lmipFb() As lmipB   ' No LMIP header is used, because the STATs chunks reveal how many textures are present.
                        '  (i.e. the sum of the number of shaders used per texture group).
                        '   The hgn_advancedarray.hod shows that an LMIP chunk does not always have to be present!!!
    multCh() As multC   ' ship type HODs contain at least one MULT chunk (except the vgr_c_innate_engine.hod).
    gobgCh() As gobgC   ' NOT ALL ship type HODs contain goblins!!
End Type

'***************************************************************************************************

Public Type nameB
    tID As String * 4   ' type ID = 'NAME'
    nN As String        ' 'Homeworld2 Multi Mesh File'
End Type

Public Type versB
    tID As String * 4   ' type ID = 'VERS'
    vNo As Long         ' version number = 131072 ? {00 02 00 00}
End Type

Public Type HODcomp
    versFh As fHdr      ' with 'FORM'
    versFb As versB
    nameFh As fHdr      ' with 'FORM'
    nameFb As nameB
    hvmdFh As fHdr      ' with 'FORM'
    hvmdFb As hvmdB
    dtrmFh As fHdr      ' with 'FORM'
    dtrmFb As dtrmB
    infoFh As fHdr      ' with 'FORM'
    infoFb As infoB
End Type

Public myHOD As HODcomp
Public noSTAThdr As Boolean     ' Used for identifying HOD types.

Public LMIPpresence As Boolean
Public ETSHpresence As Boolean
Public GLOWpresence As Boolean
Public BURNpresence As Boolean
Public NAVLpresence As Boolean
Public COLDpresence As Boolean
Public ANIMpresence As Boolean
Public MRKRpresence As Boolean
Public GOBGpresence As Boolean
Public DOCKpresence As Boolean
Public BSRMpresence As Boolean

'***************************************************************************************************

Type reportHOD
    repHdr As String
    sepr As String
    onSTAT As String
    onMULT As String
    onGOBG As String
    nrETSHs As String
    onETSH() As String
    onGLOW As String
    nrBURNs As String
    nrNAVLs As String
    onNAVL() As String
    nrDOCK As String
    onDOCK() As String
    nrANIMs As String
    onANIM() As String
    onMRKR As String
    onCOLD As String
    onBSRM As String
    repFtr As String
End Type


Public Function genHODreport()

Dim myRep As reportHOD
Dim i As Integer
Dim j As Integer
Dim k As Integer
Dim tmpInt As Integer
Dim tmpStr As String
Dim largestTag As Long
Dim tmpBol As Boolean
Dim tmpLong As Long
Dim repNo As Integer
Dim GLOWfaceCounter(4) As Long
Dim MULTfaceCounter(7) As Long
Dim dupe1 As Long
Dim dupe2 As Long

    tmpStr = openHW2shipHOD
    If tmpStr = "" Then Exit Function
    
    With myRep
        .repHdr = "{\rtf1\ansi\ansicpg1252\deff0\deflang1033{\fonttbl{\f0\fswiss\fcharset0 Arial;}}" _
                    & Chr(13) & Chr(10) & "\viewkind4\uc1\pard\f0\fs20 \par" & Chr(13) & Chr(10) _
                    & "Report on " & tmpStr & " HOD file:\par" & Chr(13) & Chr(10)
        tmpLong = ReverseLong(myHOD.versFh.lF)
        If tmpLong <> 8 Then
            .repHdr = .repHdr & "\par" & Chr(13) & Chr(10) & "Unusual HOD file length (Long, reversed): " _
                              & tmpLong & "\par" & Chr(13) & Chr(10)
        End If
        If myHOD.versFb.vNo <> 512 Then
            .repHdr = .repHdr & "\par" & Chr(13) & Chr(10) & "Unusual HOD version (Long): " _
                              & myHOD.versFb.vNo & "\par" & Chr(13) & Chr(10)
        End If
    
        ' HIER reporting:
        .repHdr = .repHdr & myHOD.dtrmFb.hierFb.noHobjects & " hierarchy objects\par" & Chr(13) & Chr(10) & "\par" & Chr(13) & Chr(10) _
                                                            & "Absent chunks: "

        ' STATs reporting:
        tmpLong = UBound(myHOD.hvmdFb.statCh)
        If noSTAThdr Then .onSTAT = "\par" & Chr(13) & Chr(10) & "HOD without STAT headers !!!\par" & Chr(13) & Chr(10)
        .onSTAT = .onSTAT & "\par" & Chr(13) & Chr(10) & "\par" & Chr(13) & Chr(10) & tmpLong & " STATs"
    End With
    tmpInt = 0
    largestTag = 0
    tmpBol = False
    For i = 0 To tmpLong
        With myHOD.hvmdFb.statCh(i).statFb
            tmpInt = tmpInt + .noShaders
            For j = 1 To .noShaders
                If .shader(j).texTag > largestTag Then
                    largestTag = .shader(j).texTag
                End If
            Next j
            If InStr(.nTgrp, "lambert") Then tmpBol = True
        End With
    Next i
    With myRep
        .onSTAT = .onSTAT & " with a total of " & tmpInt & " shaders "
        If tmpBol Then
            .onSTAT = .onSTAT & "(+ a 'lambert' shader)\par" & Chr(13) & Chr(10)
        Else
            .onSTAT = .onSTAT & "(no 'lambert' shader present)\par" & Chr(13) & Chr(10)
        End If
    End With
    
    ' LMIPs reporting:
    If LMIPpresence Then
        With myRep
            tmpLong = UBound(myHOD.hvmdFb.lmipFb)
            If (largestTag = tmpLong) And ((largestTag + 1) = tmpInt) Then
                .onSTAT = .onSTAT & " (corresponding to # LMIPs and the highest shader tag).\par" & Chr(13) & Chr(10)
            Else
                .onSTAT = .onSTAT & "Either the highest shader tag (" & largestTag _
                                  & ", first tag = 0), or the number of textures (" & (tmpLong + 1) _
                                  & " DOESN'T CORRESPOND !!!\par" & Chr(13) & Chr(10)
            End If
        End With
        tmpBol = False
        For i = 0 To tmpLong
            With myHOD.hvmdFb.lmipFb(i).dxt5Fb
                If .tID = "8888" Then tmpBol = True    ' -> should be overall project setting!...
            End With
        Next i
        
        If tmpBol Then
            myRep.onSTAT = myRep.onSTAT & "\par" & Chr(13) & Chr(10) & "The LMIP chunks contain Targa images.\par" & Chr(13) & Chr(10)
        End If
    Else
        myRep.repHdr = myRep.repHdr & "LMIP; "
    End If
    
    ' MULTs reporting:
    tmpLong = UBound(myHOD.hvmdFb.multCh)
    If tmpLong = 1 Then
        myRep.onMULT = "\par" & Chr(13) & Chr(10) & tmpLong & " MULT chunk with:\par" & Chr(13) & Chr(10)
    Else
        myRep.onMULT = "\par" & Chr(13) & Chr(10) & tmpLong & " MULTs with:\par" & Chr(13) & Chr(10)
    End If
    For i = 1 To tmpLong
        For j = 1 To myHOD.hvmdFb.multCh(i).multFb.noMeshes
            With myHOD.hvmdFb.multCh(i).multFb.bmshCh(j).bmshFb
                For k = 1 To .noMshs
                    MULTfaceCounter(.LOD) = MULTfaceCounter(.LOD) + .shipmesh(k).noFcs
                Next k
            End With
        Next j
    Next i
    With myRep
        .onMULT = .onMULT & "===============\par" & Chr(13) & Chr(10)
        For i = 0 To 4
            If MULTfaceCounter(i) <> 0 Then
                .onMULT = .onMULT & "Total faces LOD" & i & ": " & MULTfaceCounter(i) & "\par" & Chr(13) & Chr(10)
            End If
        Next i
    End With
    
    ' GOBGs reporting:
    If GOBGpresence Then
        With myRep
            tmpLong = UBound(myHOD.hvmdFb.gobgCh)
            If tmpLong = 1 Then
                .onGOBG = "\par" & Chr(13) & Chr(10) & tmpLong & " GOBG chunk with "
            Else
                .onGOBG = "\par" & Chr(13) & Chr(10) & tmpLong & " GOBGs with "
            End If
        End With
        For i = 1 To tmpLong
            With myHOD.hvmdFb.gobgCh(i).gobgFb.bmshFb
                For j = 1 To .noMshs
                    MULTfaceCounter(5) = MULTfaceCounter(5) + .shipmesh(j).noFcs
                Next j
            End With
        Next i
        myRep.onGOBG = myRep.onGOBG & MULTfaceCounter(5) & " faces in total.\par" & Chr(13) & Chr(10)
    Else
        myRep.repHdr = myRep.repHdr & "GOBG; "
    End If
    
    With myHOD.dtrmFb
        ' ETSHs reporting:
        If ETSHpresence Then
            tmpLong = UBound(.etshFb)
            myRep.nrETSHs = "\par" & Chr(13) & Chr(10) & tmpLong & " ETSHs" & "\par" & Chr(13) & Chr(10)
            ReDim myRep.onETSH(1 To tmpLong)
            For i = 1 To tmpLong
                myRep.onETSH(i) = "\par" & Chr(13) & Chr(10) & " " & i & "... with " & UBound(.etshFb(i).tSet) & " trail set(s)"
                tmpInt = 0
                For j = 1 To UBound(.etshFb(i).tSet)
                    tmpInt = tmpInt + .etshFb(i).tSet(j).noConnPnts
                Next j
                myRep.onETSH(i) = myRep.onETSH(i) & ", containing " & tmpInt & " shape points.\par" & Chr(13) & Chr(10) & "\par" & Chr(13) & Chr(10)
            Next i
        Else
            myRep.repHdr = myRep.repHdr & "ETSH; "
        End If
    End With
    
    ' GLOWs reporting:
    If GLOWpresence Then
        tmpLong = UBound(myHOD.dtrmFb.glowCh)
        With myRep
            If tmpLong = 1 Then
                .onGLOW = "\par" & Chr(10) & Chr(13) & tmpLong & " GLOW chunk with:\par" & Chr(13) & Chr(10)
            Else
                .onGLOW = "\par" & Chr(10) & Chr(13) & tmpLong & " GLOWs with:\par" & Chr(13) & Chr(10)
            End If
        End With
        For i = 1 To tmpLong
            With myHOD.dtrmFb.glowCh(i).glowFb
                For j = 1 To .bmshFb.noMshs
                    GLOWfaceCounter(.LOD) = GLOWfaceCounter(.LOD) + .bmshFb.shipmesh(j).noFcs
                Next j
            End With
        Next i
        With myRep
            For i = 0 To 4
                If GLOWfaceCounter(i) <> 0 Then
                    .onGLOW = .onGLOW & "Total faces for LOD" & i & ": " & GLOWfaceCounter(i) & "\par" & Chr(13) & Chr(10)
                End If
            Next i
        End With
    Else
        myRep.repHdr = myRep.repHdr & "GLOW; "
    End If
    
    ' BURNs reporting:
    If BURNpresence Then
        myRep.nrBURNs = "\par" & Chr(13) & Chr(10) & UBound(myHOD.dtrmFb.burnFb) & " BURNs" & "\par" & Chr(13) & Chr(10) & "\par" & Chr(13) & Chr(10)
    Else
        myRep.repHdr = myRep.repHdr & "BURN; "
    End If
        
    With myHOD.dtrmFb.navlFb
        ' NAVLs reporting:
        If NAVLpresence Then
            myRep.nrNAVLs = "\par" & Chr(13) & Chr(10) & UBound(.navl) & " Navlights" & "\par" & Chr(13) & Chr(10)
    '        ReDim myRep.onNAVL(1 To UBound(.navl))
    '        For i = 1 To UBound(.navl)
    '            myRep.onNAVL(i) = "\par" & Chr(13) & Chr(10) & .navl(i).nHo & "\tab " & .navl(i).n1 _
    '            & "\tab " & .navl(i).n2 & "\tab " & .navl(i).n3 & "\tab " & .navl(i).n4 _
    '            & "\tab " & .navl(i).n5 & "\tab " & .navl(i).n6 & "\tab " & .navl(i).n7 _
    '            & "\tab " & .navl(i).n8 & "\tab " & .navl(i).n9 & "\par" & Chr(13) & Chr(10) & "\par" & Chr(13) & Chr(10)
    '        Next i
        Else
            myRep.repHdr = myRep.repHdr & "NAVL; "
        End If
    End With
    
    With myHOD.dtrmFb.dockFb
        ' DOCKs reporting:
        If DOCKpresence Then
            myRep.nrDOCK = "\par" & Chr(13) & Chr(10) & "\par" & Chr(13) & Chr(10) & .noPaths & " DOCK paths" & "\par" & Chr(13) & Chr(10)
            ReDim myRep.onDOCK(1 To .noPaths)
            tmpBol = False
            For i = 1 To .noPaths
                myRep.onDOCK(i) = "\par" & Chr(13) & Chr(10) & .path(i).nP & "\tab " & .path(i).flag1 _
                & "\tab " & .path(i).flag2 & "\tab " & .path(i).flag3 & "\tab " & .path(i).flag4 _
                & "\tab " & .path(i).noPars & " par.'s for " & .path(i).noPpoints & " pp's, "
                If .path(i).nAPs <> "" Then myRep.onDOCK(i) = myRep.onDOCK(i) & "| " & .path(i).nAPs & " | (= alt. paths, "
                myRep.onDOCK(i) = myRep.onDOCK(i) & "for " & .path(i).nSfs & " (unknown8 = " & .path(i).unknown8 & ").\par" & Chr(13) & Chr(10)
                If .path(i).unknown8 <> 6 Then
                    tmpBol = True
                End If
            Next i
            If Not tmpBol Then Erase myRep.onDOCK
        Else
            myRep.repHdr = myRep.repHdr & "DOCK; "
        End If
    End With
    
    With myHOD.dtrmFb
        ' ANIMs reporting:
        If ANIMpresence Then
            tmpLong = UBound(.animCh)
            myRep.nrANIMs = "\par" & Chr(13) & Chr(10) & "# ANIMs: " & tmpLong & "\par" & Chr(13) & Chr(10)
            ReDim myRep.onANIM(1 To tmpLong)
            For i = 1 To tmpLong
                myRep.onANIM(i) = "\par" & Chr(13) & Chr(10) & .animCh(i).animFb.noKeyfs & " keyframes for " & .animCh(i).animFb.nPar & "\par" & Chr(13) & Chr(10)
            Next i
        Else
            myRep.repHdr = myRep.repHdr & "ANIM; "
        End If
    End With
    
    ' MRKRs reporting:
    With myRep
        If MRKRpresence Then
            tmpLong = UBound(myHOD.dtrmFb.mrkrCh)
            If tmpLong = 1 Then
                .onMRKR = "\par" & Chr(13) & Chr(10) & tmpLong & " MRKR chunk" & "\par" & Chr(13) & Chr(10)
            Else
                .onMRKR = "\par" & Chr(13) & Chr(10) & tmpLong & " MRKRs" & "\par" & Chr(13) & Chr(10)
            End If
        Else
            .repHdr = .repHdr & "MRKR; "
        End If
    
        ' COLDs reporting:
        If COLDpresence Then
            tmpLong = UBound(myHOD.dtrmFb.coldCh)
            If tmpLong = 1 Then
                .onCOLD = "\par" & Chr(13) & Chr(10) & tmpLong & " COLD chunk\tab " & "TRIS vertices and faces:\par" & Chr(13) & Chr(10)
            Else
                .onCOLD = "\par" & Chr(13) & Chr(10) & tmpLong & " COLDs\tab " & "TRIS vertices and faces:\par" & Chr(13) & Chr(10)
            End If
            For i = 1 To tmpLong
                .onCOLD = .onCOLD & myHOD.dtrmFb.coldCh(i).coldFb.trisFb.noFcs & "\tab " & myHOD.dtrmFb.coldCh(i).coldFb.nO & "\par" & Chr(13) & Chr(10)
            Next i
        Else
            .repHdr = .repHdr & "COLD; "
        End If
    End With

    ' BSRMs reporting:
    If BSRMpresence Then
        With myHOD.hvmdFb
            tmpLong = UBound(myHOD.dtrmFb.bsrmCh)
            If GOBGpresence Then
                tmpBol = (tmpLong = (UBound(.multCh) + UBound(.gobgCh)))
            Else
                tmpBol = (tmpLong = UBound(.multCh))
            End If
            If tmpBol Then
                myRep.onBSRM = "\par" & Chr(13) & Chr(10) & "# BSRMs = # MULTs + # GOBGs (OK):\par" & Chr(13) & Chr(10) _
                                & "=============================\par" & Chr(13) & Chr(10)
            Else
                myRep.onBSRM = "\par" & Chr(13) & Chr(10) & tmpLong & " BSRMs DOES NOT CORRESPOND TO # MULTs + # GOBGs !!!\par" & Chr(13) & Chr(10)
            End If
        End With
        For i = 1 To tmpLong
            With myHOD.dtrmFb.bsrmCh(i).bsrmFb
                ' TRISs reporting:
                myRep.onBSRM = myRep.onBSRM & "TRIS faces : " & .trisFb.noFcs & ";\tab "
                ' VNRMs reporting:
                myRep.onBSRM = myRep.onBSRM & "VNRM 'faces' : " & .vnrmFb.noFcs & ";\tab "
                ' SSUBs reporting:
                largestTag = 0
                tmpInt = 0
                dupe1 = .ssubFb.fcIdx(1)
                dupe2 = .ssubFb.fcIdx(2)
                For j = 1 To UBound(.ssubFb.fcIdx)
                    If .ssubFb.fcIdx(j) > largestTag Then largestTag = .ssubFb.fcIdx(j)
        '**** Special counter ****
                  '  If j > 2 Then
                  '      If (.ssubFb.fcIdx(j) <> dupe1) And (.ssubFb.fcIdx(j) <> dupe2) Then tmpInt = tmpInt + 1
                  '      dupe1 = .ssubFb.fcIdx(j - 1)
                  '      dupe2 = .ssubFb.fcIdx(j)
                  '  End If
        '**** End special counter ****
                Next j
            '    MsgBox tmpInt
                If ((largestTag + 1) = .trisFb.noFcs) And ((largestTag + 1) = .vnrmFb.noFcs) Then
                    myRep.onBSRM = myRep.onBSRM & "amount of SSUB face indices : " & UBound(.ssubFb.fcIdx) & " ; flags: " _
                                    & .ssubFb.flag(1) & "\tab " & .ssubFb.flag(2) & "\tab " & .ssubFb.flag(3) & "\tab " & .ssubFb.flag(4)
                Else
                    myRep.onBSRM = myRep.onBSRM & "amount of SSUB face indices : " & UBound(.ssubFb.fcIdx) & " largest tag " & largestTag _
                                                & " NOT EQUAL to VNRM or TRIS faces!!!); flags : " & .ssubFb.flag(1) & "\tab " & .ssubFb.flag(2) _
                                                & "\tab " & .ssubFb.flag(3) & "\tab " & .ssubFb.flag(4)
                End If
                ' DESCs reporting:
                If .descFb.nonGoblin Then
                    myRep.onBSRM = myRep.onBSRM & " -> " & .descFb.nM & ";\par" & Chr(13) & Chr(10)
                    MULTfaceCounter(6) = MULTfaceCounter(6) + (largestTag + 1)
                Else
                    myRep.onBSRM = myRep.onBSRM & " -> " & .descFb.nM & " = goblin mesh ( flag = " & .descFb.nonGoblin & " );\par" & Chr(13) & Chr(10)
                    MULTfaceCounter(7) = MULTfaceCounter(7) + (largestTag + 1)
                End If
            End With
            
            MsgBox listFacesPerVertex(i)
            
        Next i
        
        ' Special faces cross check:
        If MULTfaceCounter(0) <> MULTfaceCounter(6) Then myRep.repFtr = "\par" & Chr(13) & Chr(10) & "\par" & Chr(13) & Chr(10) _
                                                                      & "**** The sum of all of the highest face indices of all non-goblin SSUB chunks" _
                                                                          & " DOES NOT CORRESPOND to the sum of all LOD0 faces in all (MULT) BMSH chunks ****"
        If MULTfaceCounter(5) <> MULTfaceCounter(7) Then myRep.repFtr = myRep.repFtr & "\par" & Chr(13) & Chr(10) _
                                                                     & "******** The sum of all of the highest face indices of all GOBLIN SSUB chunks" _
                                                                          & " DOES NOT CORRESPOND to the sum of all faces in all (GOBG) BMSH chunks ********"
        
    Else
        myRep.repHdr = myRep.repHdr & "\par" & Chr(13) & Chr(10) & "BSRM; "
    End If
        
    myRep.repFtr = myRep.repFtr & "\par" & Chr(13) & Chr(10) & "}" & "\par" & Chr(13) & Chr(10) & " "
    
    repNo = FreeFile
    Open App.path & "\HODreport.rtf" For Binary As #repNo
    Put #repNo, , myRep
    Close #repNo

End Function

Private Function listFacesPerVertex(noBmsh) As Integer

Dim i As Integer
Dim j As Integer
Dim facesListNo As Integer
Dim matchVtx As Long
Dim listArray() As Integer

    ReDim listArray(0)
    
    matchVtx = 0
    With myHOD.dtrmFb.bsrmCh(noBmsh).bsrmFb.trisFb
        For i = 0 To (.noVtcs - 1)
            For j = 1 To .noFcs
                If .fc(j).vtx1 = i Then
                    listArray(matchVtx) = j
                    matchVtx = matchVtx + 1
                    ReDim Preserve listArray(matchVtx)
                End If
                If .fc(j).vtx2 = i Then
                    listArray(matchVtx) = j
                    matchVtx = matchVtx + 1
                    ReDim Preserve listArray(matchVtx)
                End If
                If .fc(j).vtx3 = i Then
                    listArray(matchVtx) = j
                    matchVtx = matchVtx + 1
                    ReDim Preserve listArray(matchVtx)
                End If
            Next j
        Next i
    End With
    
    listFacesPerVertex = (matchVtx - 1)
    
'    facesListNo = FreeFile
'    Open "BMSHfacesList" & noBmsh & ".bin" For Binary As #facesListNo
'    Put #facesListNo, , listArray
'    Close #facesListNo
        
End Function

Private Function openHW2shipHOD() As String

Dim dirAndFileName As String

    With frmConfigure.CommonDialog1
        ' Set CancelError is True
        .CancelError = False
        ' Set flags
        .flags = cdlOFNHideReadOnly
        ' Set filters
        .Filter = "Homeworld2 (ship type) HOD files (*.hod)|*.hod"
        .InitDir = App.path
        .ShowOpen
        If (.fileName <> "") And (InStr(.fileName, ".hod") <> 0) Then
            dirAndFileName = .fileName
            openHW2shipHOD = Mid$(.fileName, 1, ((InStr(.fileName, ".hod")) - 1))
            openHW2shipHOD = Mid$(openHW2shipHOD, ((InStrRev(openHW2shipHOD, "\")) + 1), Len(openHW2shipHOD))
            importHW2shipHOD (dirAndFileName)
        Else
            MsgBox "Please select an Homeworld2 ship HOD file."
        End If
        
    End With

End Function

Public Sub importHW2shipHOD(shipHODdirANDfileName As String)

Dim myHODno As Integer
Dim i As Integer
Dim k As Integer
Dim tmpByte As Byte
Dim tmpStr1 As String * 4
Dim tmpStr2 As String * 4
Dim tmpLong As Long      ' Don't merge with other Long definitions!
Dim j As Long
Dim l As Long
Dim checkBytes As String
Dim readStr2 As Boolean

    noSTAThdr = False
    readStr2 = True
    
    LMIPpresence = False
    ETSHpresence = False
    GLOWpresence = False
    BURNpresence = False
    NAVLpresence = False
    COLDpresence = False
    ANIMpresence = False
    MRKRpresence = False
    GOBGpresence = False
    DOCKpresence = False
    BSRMpresence = False
    
    myHODno = FreeFile()
    Open shipHODdirANDfileName For Binary Access Read As myHODno
    
    If LOF(myHODno) > 0 Then
        With myHOD
            Get myHODno, , .versFh
            Get myHODno, , .versFb
            Get myHODno, , .nameFh
            .nameFb.nN = Space(ReverseLong(.nameFh.lF) - 4)  ' Formatting the string before reading the name.
            Get myHODno, , .nameFb.tID
            Get myHODno, , .nameFb.nN
            If .nameFb.nN <> "Homeworld2 Multi Mesh File" Then
                MsgBox "Not a valid HW2 ship type HOD file, please try another."
                Exit Sub
            End If
            Get myHODno, , .hvmdFh
            Get myHODno, , .hvmdFb.tID
            If .hvmdFb.tID <> "HVMD" Then
                MsgBox "Not a valid HW2 ship type HOD file, please try another."
                Exit Sub
            End If
        End With
        
        With myHOD.hvmdFb
            i = 1
            Do
                Get myHODno, , tmpStr1
    '**** This probably has to become a 'Select Case' study, because also LMIPs can be absent !!!
                If tmpStr1 <> "STAT" Then       ' Made because of the 'cap_chunk_01.hod', which doesn't have STAT headers (!).
                    If tmpStr1 <> "NRML" Then   '  (hgn_assaultcorvetteelite.hod & hgn_attackbomberelite.hod too)
                        Exit Do                 '    This is a problem, because they look like those mission sphere meshes...
                    Else
                        Get myHODno, , tmpLong
                        Get myHODno, , tmpStr2
                        Select Case tmpStr2
                        Case "STAT"
                            noSTAThdr = False
                            ReDim Preserve .statCh(i)
                            .statCh(i).statFh.tF = "NRML"
                            .statCh(i).statFh.lF = tmpLong
                            .statCh(i).statFb.tID = "STAT"
                        Case "MULT"
                            LMIPpresence = False
                            readStr2 = False
                            Exit Do
                        Case Else
                            Exit Do
                        End Select
                        Get myHODno, , .statCh(i).statFb.altLf
                    End If
                Else
                    noSTAThdr = True
                    ReDim Preserve .statCh(i)
                    .statCh(i).statFh.tF = "NRML"
                    .statCh(i).statFb.tID = "STAT"
                    Get myHODno, , .statCh(i).statFb.altLf
                    .statCh(i).statFh.lF = .statCh(i).statFb.altLf
                    .statCh(i).statFb.altLf = ReverseLong(.statCh(i).statFb.altLf)
                End If
                Get myHODno, , .statCh(i).statFb.lTgrpN
                .statCh(i).statFb.nTgrp = Space(.statCh(i).statFb.lTgrpN)  ' Formatting the string before reading the name.
                Get myHODno, , .statCh(i).statFb.nTgrp
                Get myHODno, , .statCh(i).statFb.lType
                .statCh(i).statFb.nType = Space(.statCh(i).statFb.lType)
                Get myHODno, , .statCh(i).statFb.nType
                Get myHODno, , .statCh(i).statFb.noShaders
                If .statCh(i).statFb.noShaders > 10 Then ' Actually probably also the 'noSTAThdr = True' case...
                    ReDim Preserve .statCh(i).statFb.shader(2)
                    For j = 0 To 1
                        ' Rebuild the shaders...
                        .statCh(i).statFb.shader(j).unknown1 = 5
                        .statCh(i).statFb.shader(j).unknown2 = 4
                        .statCh(i).statFb.shader(j).texTag = (2 * (i - 1)) + j
                        .statCh(i).statFb.shader(j).lShadN = 8
                        If .statCh(i).statFb.shader(j).texTag > 9 Then
                            .statCh(i).statFb.shader(j).nShad = "shader" & .statCh(i).statFb.shader(j).texTag
                        Else
                            .statCh(i).statFb.shader(j).nShad = "shader0" & .statCh(i).statFb.shader(j).texTag
                        End If
                    Next j
                    tmpLong = Seek(myHODno)
                    Seek myHODno, (tmpLong + .statCh(i).statFb.altLf - 12 - .statCh(i).statFb.lType - .statCh(i).statFb.lTgrpN)
                Else
                    For j = 1 To .statCh(i).statFb.noShaders
                        ReDim Preserve .statCh(i).statFb.shader(1 To j) ' Do not define before the 'For...to' loop -> will go wrong.
                        Get myHODno, , .statCh(i).statFb.shader(j).unknown1
                        Get myHODno, , .statCh(i).statFb.shader(j).unknown2
                        Get myHODno, , .statCh(i).statFb.shader(j).texTag
                        If noSTAThdr And (.statCh(i).statFb.shader(j).texTag = 0) Then
                            .statCh(i).statFb.shader(j).texTag = 2 * i
                        End If
                        If Not noSTAThdr Then
                            Get myHODno, , .statCh(i).statFb.shader(j).lShadN
                            .statCh(i).statFb.shader(j).nShad = Space(.statCh(i).statFb.shader(j).lShadN)
                            Get myHODno, , .statCh(i).statFb.shader(j).nShad
                        End If
                    Next j
                End If
                i = i + 1
            Loop
            
            i = 0         ' Starting from zero to follow the 'texTag' indices.
            Do
                If tmpStr1 <> "LMIP" Then Exit Do
                LMIPpresence = True
                ReDim Preserve .lmipFb(i)
                .lmipFb(i).tID = tmpStr1
                Get myHODno, , .lmipFb(i).lF
                Get myHODno, , .lmipFb(i).lFDN
                .lmipFb(i).nFDN = Space(.lmipFb(i).lFDN)    ' Formatting the string before reading the name.
                Get myHODno, , .lmipFb(i).nFDN
                Get myHODno, , .lmipFb(i).dxt5Fb.tID        ' "DXT5" for DDS, "8888" for TGA (if TGA then the code following is still OK :-)
                Get myHODno, , .lmipFb(i).dxt5Fb.noMIPls
                Get myHODno, , .lmipFb(i).dxt5Fb.Wi
                Get myHODno, , .lmipFb(i).dxt5Fb.Hi
                tmpLong = ReverseLong(.lmipFb(i).lF) - .lmipFb(i).lFDN - 20
                ReDim .lmipFb(i).dxt5Fb.Body(1 To tmpLong)
                Get myHODno, , .lmipFb(i).dxt5Fb.Body
                i = i + 1
                Get myHODno, , tmpStr1
            Loop
            
            i = 1
            Do
                
                If readStr2 Then
                    Get myHODno, , tmpLong
                    Get myHODno, , tmpStr2
                End If
                
                If Not ((tmpStr1 = "NRML") And (tmpStr2 = "MULT")) Then Exit Do
                ReDim Preserve .multCh(i)
                .multCh(i).multFh.tF = "NRML"
                .multCh(i).multFh.lF = tmpLong
                .multCh(i).multFb.tID = tmpStr1
                Get myHODno, , .multCh(i).multFb.altLf
                Get myHODno, , .multCh(i).multFb.lN
                .multCh(i).multFb.nM = Space(.multCh(i).multFb.lN)  ' Formatting the string before reading the name.
                Get myHODno, , .multCh(i).multFb.nM
                Get myHODno, , .multCh(i).multFb.lO
                .multCh(i).multFb.nO = Space(.multCh(i).multFb.lO)  ' Formatting the string before reading the name.
                Get myHODno, , .multCh(i).multFb.nO
                Get myHODno, , .multCh(i).multFb.noMeshes
                ReDim .multCh(i).multFb.bmshCh(1 To .multCh(i).multFb.noMeshes)
                For j = 1 To .multCh(i).multFb.noMeshes
                    Get myHODno, , .multCh(i).multFb.bmshCh(j).bmshFh
                    Get myHODno, , .multCh(i).multFb.bmshCh(j).bmshFb.tID
                    If .multCh(i).multFb.bmshCh(j).bmshFb.tID <> "BMSH" Then
                        MsgBox "Not a valid mesh: " & j & " (file pos. " & Seek(myHODno) & ")."
                        Exit Do
                    End If
                    Get myHODno, , .multCh(i).multFb.bmshCh(j).bmshFb.altLf
                    Get myHODno, , .multCh(i).multFb.bmshCh(j).bmshFb.LOD
                    Get myHODno, , .multCh(i).multFb.bmshCh(j).bmshFb.noMshs
                    ReDim .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(1 To .multCh(i).multFb.bmshCh(j).bmshFb.noMshs)
                    For k = 1 To .multCh(i).multFb.bmshCh(j).bmshFb.noMshs
                        Get myHODno, , .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).texTag
                        Get myHODno, , .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).dwordsPerVtx
                        Get myHODno, , .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).noVtcs
                        ReDim .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).shipVtx(1 To .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).noVtcs)
                        Get myHODno, , .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).shipVtx
                        Get myHODno, , .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).sepr
                        checkBytes = ""
                        For l = 1 To 6
                            checkBytes = checkBytes & .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).sepr(l)
                        Next l
                        If checkBytes <> "102200" Then
                            If checkBytes <> "17306200" Then
                                MsgBox "The six bytes between vertices & faces are: " & checkBytes & " (file pos " & Seek(myHODno) & ")."
                            End If
                        End If
                        If noSTAThdr Then
                            Exit Do         ' Possibly now Voronoi meshes (polygons with > 3 vertices...).
                        Else
                            Get myHODno, , .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).noFcs
                            .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).noFcs = .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).noFcs / 3
                            ReDim .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).shipFc(1 To .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).noFcs)
                            Get myHODno, , .multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).shipFc
                        End If
                    Next k
                Next j
                i = i + 1
                Get myHODno, , tmpStr1
                readStr2 = True
            Loop
            
            i = 1
            Do
                If Not ((tmpStr1 = "NRML") And (tmpStr2 = "GOBG")) Then Exit Do
                GOBGpresence = True
                ReDim Preserve .gobgCh(i)
                .gobgCh(i).gobgFh.tF = "NRML"
                .gobgCh(i).gobgFh.lF = tmpLong
                .gobgCh(i).gobgFb.tID = tmpStr1
                Get myHODno, , .gobgCh(i).gobgFb.altLf
                Get myHODno, , .gobgCh(i).gobgFb.lN
                .gobgCh(i).gobgFb.nG = Space(.gobgCh(i).gobgFb.lN)  ' Formatting the string before reading the name.
                Get myHODno, , .gobgCh(i).gobgFb.nG
                Get myHODno, , .gobgCh(i).gobgFb.lO
                .gobgCh(i).gobgFb.nO = Space(.gobgCh(i).gobgFb.lO)  ' Formatting the string before reading the name.
                Get myHODno, , .gobgCh(i).gobgFb.nO
                Get myHODno, , .gobgCh(i).gobgFb.bmshFh
                Get myHODno, , .gobgCh(i).gobgFb.bmshFb.tID
                If .gobgCh(i).gobgFb.bmshFb.tID <> "BMSH" Then
                    MsgBox "Not a valid mesh: " & j & " (file pos. " & Seek(myHODno) & ")."
                    Exit Do
                End If
                Get myHODno, , .gobgCh(i).gobgFb.bmshFb.altLf
                Get myHODno, , .gobgCh(i).gobgFb.bmshFb.LOD
                Get myHODno, , .gobgCh(i).gobgFb.bmshFb.noMshs
                ReDim .gobgCh(i).gobgFb.bmshFb.shipmesh(1 To .gobgCh(i).gobgFb.bmshFb.noMshs)
                For j = 1 To .gobgCh(i).gobgFb.bmshFb.noMshs
                    Get myHODno, , .gobgCh(i).gobgFb.bmshFb.shipmesh(j).texTag
                    Get myHODno, , .gobgCh(i).gobgFb.bmshFb.shipmesh(j).dwordsPerVtx
                    Get myHODno, , .gobgCh(i).gobgFb.bmshFb.shipmesh(j).noVtcs
                    ReDim .gobgCh(i).gobgFb.bmshFb.shipmesh(j).shipVtx(1 To .gobgCh(i).gobgFb.bmshFb.shipmesh(j).noVtcs)
                    Get myHODno, , .gobgCh(i).gobgFb.bmshFb.shipmesh(j).shipVtx
                    Get myHODno, , .gobgCh(i).gobgFb.bmshFb.shipmesh(j).sepr
                    checkBytes = ""
                    For k = 1 To 6
                        checkBytes = checkBytes & .gobgCh(i).gobgFb.bmshFb.shipmesh(j).sepr(k)
                    Next k
                    If checkBytes <> "102200" Then MsgBox "The six bytes between vertices & faces are different: " & checkBytes & " (file pos " & Seek(myHODno) & ")."
                    Get myHODno, , .gobgCh(i).gobgFb.bmshFb.shipmesh(j).noFcs
                    .gobgCh(i).gobgFb.bmshFb.shipmesh(j).noFcs = .gobgCh(i).gobgFb.bmshFb.shipmesh(j).noFcs / 3
                    ReDim .gobgCh(i).gobgFb.bmshFb.shipmesh(j).shipFc(1 To .gobgCh(i).gobgFb.bmshFb.shipmesh(j).noFcs)
                    Get myHODno, , .gobgCh(i).gobgFb.bmshFb.shipmesh(j).shipFc
                Next j
                i = i + 1
                
                Get myHODno, , tmpStr1
                Get myHODno, , tmpLong
                Get myHODno, , tmpStr2
            Loop
            
        End With
        
        If (tmpStr1 = "FORM") And (tmpStr2 = "DTRM") Then
            myHOD.dtrmFh.tF = "FORM"
            myHOD.dtrmFh.lF = tmpLong
            
            With myHOD.dtrmFb
            
                .tID = tmpStr2
                Get myHODno, , .hierFb.tID
                If .hierFb.tID = "HIER" Then
                    Get myHODno, , .hierFb.lF
                    Get myHODno, , .hierFb.noHobjects
                    ReDim .hierFb.hObj(.hierFb.noHobjects)
                    For i = 1 To .hierFb.noHobjects
                        Get myHODno, , .hierFb.hObj(i).lO
                        .hierFb.hObj(i).nO = Space(.hierFb.hObj(i).lO)  ' Formatting the string before reading the name.
                        Get myHODno, , .hierFb.hObj(i).nO
                        Get myHODno, , .hierFb.hObj(i).lP               ' It doesn't matter if the length of the parent name is zero.
                        .hierFb.hObj(i).nP = Space(.hierFb.hObj(i).lP)  ' Formatting the string before reading the name.
                        Get myHODno, , .hierFb.hObj(i).nP
                        Get myHODno, , .hierFb.hObj(i).cldPos
                        Get myHODno, , .hierFb.hObj(i).cldRot
                        Get myHODno, , .hierFb.hObj(i).cldScl
                        Get myHODno, , .hierFb.hObj(i).cldWrl
                        Get myHODno, , .hierFb.hObj(i).sepr
                        For j = 1 To 3
                            If Not ((.hierFb.hObj(i).sepr(j) = 1) Or (.hierFb.hObj(i).sepr(j) = 0)) Then MsgBox "Separator byte " & j & " between hierarchy objects is different:" & .hierFb.hObj(i).sepr(j) & " (file pos " & Seek(myHODno) & ")."
                        Next j
                    Next i
                Else
                    MsgBox "Try to resync? (Current position in file is " & Seek(myHODno) & ")."
                End If
                
                Get myHODno, , tmpStr1
                
                Do
                    
                    Select Case tmpStr1
                    
                    Case "ETSH"
                    
                        ETSHpresence = True
                        i = 1
                        Do
                            readEtsh myHODno, i                     ' Call function for reading ETSH chunks.
                            Get myHODno, , tmpStr1
                            If tmpStr1 <> "ETSH" Then Exit Do
                            i = i + 1
                        Loop
                        
                    Case "FORM" ' ... then either GLOW, MRKR, COLD, BSRM, or INFO (final chunk).
                        
                        If readStr2 Then
                            Get myHODno, , tmpLong
                            Get myHODno, , tmpStr2
                        End If
                        
                        Select Case tmpStr2
                        
                        Case "GLOW"
                            
                            GLOWpresence = True
                            i = 1
                            Do
                                readGlow myHODno, i, tmpLong        ' Call function for reading GLOW chunks.
                                Get myHODno, , tmpStr1
                                If tmpStr1 <> "FORM" Then
                                    readStr2 = True
                                    Exit Do
                                End If
                                Get myHODno, , tmpLong
                                Get myHODno, , tmpStr2
                                If tmpStr2 <> "GLOW" Then
                                    readStr2 = False
                                    Exit Do
                                End If
                                i = i + 1
                            Loop
                            
                        Case "MRKR"
                            
                            MRKRpresence = True
                            i = 1
                            Do
                                readMrkr myHODno, i, tmpLong        ' Call function for reading MRKR chunks.
                                Get myHODno, , tmpStr1
                                If tmpStr1 <> "FORM" Then
                                    readStr2 = True
                                    Exit Do
                                End If
                                Get myHODno, , tmpLong
                                Get myHODno, , tmpStr2
                                If tmpStr2 <> "MRKR" Then
                                    readStr2 = False
                                    Exit Do
                                End If
                                i = i + 1
                            Loop
                            
                        Case "COLD"
                            
                            COLDpresence = True
                            i = 1
                            Do
                                readCold myHODno, i, tmpLong        ' Call function for reading COLD chunks (only ONCE).
                                Get myHODno, , tmpStr1
                                If tmpStr1 <> "FORM" Then
                                    readStr2 = True
                                    Exit Do
                                End If
                                Get myHODno, , tmpLong
                                Get myHODno, , tmpStr2
                                If tmpStr2 <> "COLD" Then
                                    readStr2 = False
                                    Exit Do
                                End If
                                i = i + 1
                            Loop
                            
                        Case "BSRM"
                            
                            BSRMpresence = True
                            i = 1
                            Do
                                readBsrm myHODno, i, tmpLong        ' Call function for reading BSRM chunks.
                                Get myHODno, , tmpStr1
                                If tmpStr1 <> "FORM" Then
                                    readStr2 = True
                                    Exit Do
                                End If
                                Get myHODno, , tmpLong
                                Get myHODno, , tmpStr2
                                If tmpStr2 <> "BSRM" Then
                                    readStr2 = False
                                    Exit Do
                                End If
                                i = i + 1
                            Loop
                            
'***************************************************************************************************
' BUILT IN CHECK FOR NUMBER OF BMSH CHUNKS !!
                            If GOBGpresence Then
                                If i <> (UBound(myHOD.hvmdFb.multCh) + UBound(myHOD.hvmdFb.gobgCh)) Then
                                    MsgBox "The number of (MULTs + GOBGs) does not match the number of BSRM chunks! " _
                                            & UBound(myHOD.hvmdFb.multCh) & " - " & UBound(myHOD.hvmdFb.gobgCh) & " - " & i
                                End If
                            Else
                                If i <> UBound(myHOD.hvmdFb.multCh) Then
                                    MsgBox "The number of (MULTs + GOBGs) does not match the number of BSRM chunks! " _
                                            & UBound(myHOD.hvmdFb.multCh) & " - 0 - " & i
                                End If
                            End If
' (end check).
'***************************************************************************************************
                            
                        Case "INFO"                                 ' Read final 'INFO' chunk.
                            
                            myHOD.infoFh.tF = "FORM"
                            myHOD.infoFh.lF = tmpLong
                            myHOD.infoFb.tID = "INFO"
                            Get myHODno, , myHOD.infoFb.ownrFb.tID
                            Get myHODno, , myHOD.infoFb.ownrFb.lF
                            Get myHODno, , myHOD.infoFb.ownrFb.lN
                            myHOD.infoFb.ownrFb.nOw = Space(myHOD.infoFb.ownrFb.lN)
                            Get myHODno, , myHOD.infoFb.ownrFb.nOw
                            'MsgBox "Presented to you by " & myHOD.infoFb.ownrFb.nOw
                            Exit Do
                            
                        Case Else
                            
                            Exit Do
                            
                        End Select
                        
                    Case "BURN"
                        
                        BURNpresence = True
                        i = 1
                        Do
                            readBurn myHODno, i                     ' Call function for reading BURN chunks.
                            Get myHODno, , tmpStr1
                            If tmpStr1 <> "BURN" Then Exit Do
                            i = i + 1
                        Loop
                        
                    Case "NRML" ' ... then either NAVL or ANIM (others?).
                        
                        If readStr2 Then
                            Get myHODno, , tmpLong
                            Get myHODno, , tmpStr2
                        End If
                        
                        Select Case tmpStr2
                        
                        Case "NAVL"
                            
                            NAVLpresence = True
                            readNavl myHODno, tmpLong               ' Call function for reading NAVL chunks (only ONCE).
                            Get myHODno, , tmpStr1
                            readStr2 = True
                            
                        Case "ANIM"
                            
                            ANIMpresence = True
                            i = 1
                            Do
                                readAnim myHODno, i, tmpLong        ' Call function for reading ANIM chunks.
                                Get myHODno, , tmpStr1
                                If tmpStr1 <> "NRML" Then
                                    readStr2 = True
                                    Exit Do
                                End If
                                Get myHODno, , tmpLong
                                Get myHODno, , tmpStr2
                                If tmpStr2 <> "ANIM" Then
                                    readStr2 = False
                                    Exit Do
                                End If
                                i = i + 1
                            Loop
                            
                        Case Else
                            
                            Exit Do
                            
                        End Select
                        
                    Case "DOCK"
                        
                        DOCKpresence = True
                        readDock myHODno                            ' Call function for reading DOCK chunks (only ONCE).
                        Get myHODno, , tmpStr1
                        
                    Case "BNDV"
                        
                        readBndv myHODno                            ' Call function for reading BNDV chunks (only ONCE).
                        Get myHODno, , tmpStr1
                        
                    Case Else
                        
                        Exit Do
                        
                    End Select
                    
                    If EOF(myHODno) Then Exit Do
                    
                Loop
                
            End With
            
        Else
            MsgBox "Sync lost (at file position " & Seek(myHODno) & "). Exiting..."
        End If
        
    Else
        MsgBox "Not a valid HOD file, this one's empty."
    End If
    
    Close myHODno
    
End Sub

Private Sub readEtsh(openHODno As Integer, noEtsh As Integer)

Dim j As Integer
Dim etshFileNo As Integer
Dim tmpLong3 As Long
Dim tmpLong4 As Long
Dim k As Integer
Dim m As Integer

    ReDim Preserve myHOD.dtrmFb.etshFb(1 To noEtsh)

    With myHOD.dtrmFb.etshFb(noEtsh)
    
        .tID = "ETSH"
        Get openHODno, , .lF
        Get openHODno, , .lN
        .nS = Space(.lN)
        Get openHODno, , .nS
        Get openHODno, , .lHo
        .nHo = Space(.lHo)
        Get openHODno, , .nHo
        Get openHODno, , tmpLong4
        k = 1                       ' With the hgn_pulsarcorvette each ETSH chunk consists of two parts:
        While tmpLong4 < 20         '  a problem would arise if an ETSH chunk consists of > 20 points (!).
            ReDim Preserve .tSet(1 To k)
            .tSet(k).noConnPnts = tmpLong4
            ReDim .tSet(k).cPnt(1 To .tSet(k).noConnPnts)
            Get openHODno, , .tSet(k).cPnt
            k = k + 1
            tmpLong3 = Seek(openHODno)
            Get openHODno, , tmpLong4
        Wend
        Seek openHODno, tmpLong3
        
        ' Output in order to check points in drawing program.
        'etshFileNo = FreeFile
        'Open "etshCpoints.obj" For Append As #etshFileNo
        'Print #etshFileNo, ""
        'Print #etshFileNo, "g " & .nS
        'For m = 1 To (k - 1)
        '    For j = 1 To .tSet(m).noConnPnts
        '        Print #etshFileNo, "v " & .tSet(m).cPnt(j).x & " " & .tSet(m).cPnt(j).y & " " & .tSet(m).cPnt(j).z
        '    Next j
        '    Print #etshFileNo, "# " & .tSet(m).noConnPnts & " vertices"
        'Next m
        'Print #etshFileNo, ""
        'Close etshFileNo
        ' End of output section.
        
    End With
    
End Sub

Private Sub readGlow(openHODno As Integer, noGlow As Integer, lFg As Long)

Dim j As Integer
Dim k As Long
Dim chkBts As String

    ReDim Preserve myHOD.dtrmFb.glowCh(1 To noGlow)

    With myHOD.dtrmFb.glowCh(noGlow)
    
        .glowFh.tF = "FORM"
        .glowFh.lF = lFg
        .glowFb.tID = "GLOW"
        Get openHODno, , .glowFb.tFi
        Get openHODno, , .glowFb.lFi
        Get openHODno, , .glowFb.tIDi
        If .glowFb.tIDi <> "INFO" Then
            MsgBox "No INFO chunk? " & Seek(openHODno)
            Exit Sub
        End If
        Get openHODno, , .glowFb.altLfi
        Get openHODno, , .glowFb.lN
        .glowFb.nGl = Space(.glowFb.lN)
        Get openHODno, , .glowFb.nGl
        Get openHODno, , .glowFb.lHo
        .glowFb.nHo = Space(.glowFb.lHo)
        Get openHODno, , .glowFb.nHo
        Get openHODno, , .glowFb.LOD
        Get openHODno, , .glowFb.bmshFh
        Get openHODno, , .glowFb.bmshFb.tID
        If .glowFb.bmshFb.tID <> "BMSH" Then
            MsgBox "No BMSH chunk? " & Seek(openHODno)
            Exit Sub
        End If
        Get openHODno, , .glowFb.bmshFb.altLf
        Get openHODno, , .glowFb.bmshFb.LOD
        Get openHODno, , .glowFb.bmshFb.noMshs
        ReDim .glowFb.bmshFb.shipmesh(1 To .glowFb.bmshFb.noMshs)
        For j = 1 To .glowFb.bmshFb.noMshs
            Get openHODno, , .glowFb.bmshFb.shipmesh(j).texTag
            Get openHODno, , .glowFb.bmshFb.shipmesh(j).dwordsPerVtx
            Get openHODno, , .glowFb.bmshFb.shipmesh(j).noVtcs
            ReDim .glowFb.bmshFb.shipmesh(j).shipVtx(1 To .glowFb.bmshFb.shipmesh(j).noVtcs)
            Get openHODno, , .glowFb.bmshFb.shipmesh(j).shipVtx
            Get openHODno, , .glowFb.bmshFb.shipmesh(j).sepr
            chkBts = ""
            For k = 1 To 6
                chkBts = chkBts & .glowFb.bmshFb.shipmesh(j).sepr(k)
            Next k
            If chkBts <> "102200" Then MsgBox "The six bytes between vertices & faces are different: " & chkBts
            Get openHODno, , .glowFb.bmshFb.shipmesh(j).noFcs
            .glowFb.bmshFb.shipmesh(j).noFcs = .glowFb.bmshFb.shipmesh(j).noFcs / 3
            ReDim .glowFb.bmshFb.shipmesh(j).shipFc(1 To .glowFb.bmshFb.shipmesh(j).noFcs)
            Get openHODno, , .glowFb.bmshFb.shipmesh(j).shipFc
        Next j
        
    End With


End Sub

Private Sub readNavl(openHODno As Integer, lFg As Long)
    
Dim j As Integer
Dim navlFileNo As Integer
    
    With myHOD.dtrmFb.navlFh
        .tF = "NRML"
        .lF = lFg
    End With
    
    With myHOD.dtrmFb.navlFb
    
        .tID = "NAVL"
        Get openHODno, , .altLf
        Get openHODno, , .noNavls
        ReDim .navl(1 To .noNavls)
        For j = 1 To .noNavls
            Get openHODno, , .navl(j).lHo
            .navl(j).nHo = Space(.navl(j).lHo)
            Get openHODno, , .navl(j).nHo
            Get openHODno, , .navl(j).n1
            Get openHODno, , .navl(j).n2
            Get openHODno, , .navl(j).n3
            Get openHODno, , .navl(j).n4
            Get openHODno, , .navl(j).lT
            .navl(j).tID = Space(.navl(j).lT)
            Get openHODno, , .navl(j).tID
            Get openHODno, , .navl(j).n5
            Get openHODno, , .navl(j).n6
            Get openHODno, , .navl(j).n7
            Get openHODno, , .navl(j).n8
            Get openHODno, , .navl(j).n9
            Get openHODno, , .navl(j).sepr(1)
            Get openHODno, , .navl(j).sepr(2)
            If Not (((.navl(j).sepr(1) & .navl(j).sepr(2)) = "11") Or ((.navl(j).sepr(1) & .navl(j).sepr(2)) = "00") Or ((.navl(j).sepr(1) & .navl(j).sepr(2)) = "10")) Then
                MsgBox "The merged navlight 'sepr' bytes are not equal to '11' or '00': " & (.navl(j).sepr(1) & .navl(j).sepr(2)) & " (file pos. " & Seek(openHODno) & ")."
            End If
        Next j
        
    End With
    
End Sub

Private Sub readDock(openHODno As Integer)

Dim j As Integer
Dim k As Integer
'Dim l As Integer
'Dim pathFileNo As Integer

    With myHOD.dtrmFb.dockFb
        
        .tID = "DOCK"
        Get openHODno, , .lF
        Get openHODno, , .noPaths
        ReDim .path(1 To .noPaths)
        For j = 1 To .noPaths
            Get openHODno, , .path(j).lN
            .path(j).nP = Space(.path(j).lN)
            Get openHODno, , .path(j).nP
            Get openHODno, , .path(j).lO
            .path(j).nO = Space(.path(j).lO)
            Get openHODno, , .path(j).nO
            Get openHODno, , .path(j).unknown8
            Get openHODno, , .path(j).noPars
            Get openHODno, , .path(j).flag1
            Get openHODno, , .path(j).flag2
            Get openHODno, , .path(j).flag3
            Get openHODno, , .path(j).lNSfs
            .path(j).nSfs = Space(.path(j).lNSfs)
            Get openHODno, , .path(j).nSfs
            Get openHODno, , .path(j).flag4
            Get openHODno, , .path(j).lAPs
            .path(j).nAPs = Space(.path(j).lAPs)
            Get openHODno, , .path(j).nAPs
            Get openHODno, , .path(j).noPpoints
            ReDim .path(j).pP(1 To .path(j).noPpoints)
            For k = 1 To .path(j).noPpoints
                Get openHODno, , .path(j).pP(k).pCo
                Get openHODno, , .path(j).pP(k).pOr
                Get openHODno, , .path(j).pP(k).flag0
                Get openHODno, , .path(j).pP(k).flag1
                Get openHODno, , .path(j).pP(k).flag2
                Get openHODno, , .path(j).pP(k).flag3
                ReDim .path(j).pP(k).flag4(1 To (.path(j).noPars - 4))
                Get openHODno, , .path(j).pP(k).flag4
            Next k
        Next j
        
        '  Output in order to check points in drawing program.
        'pathFileNo = FreeFile
        'For j = 1 To .noPaths
        '    Open "path" & j & ".obj" For Append As #pathFileNo
        '    Print #pathFileNo, ""
        '    Print #pathFileNo, "# path '" & .path(j).nP & "' connected to " & .path(j).nO
        '    Print #pathFileNo, "g " & .path(j).nP
        '    For k = 1 To .path(j).noPpoints
        '        Print #pathFileNo, "v " & .path(j).pP(k).pCo.x & " " & .path(j).pP(k).pCo.y & " " & .path(j).pP(k).pCo.z
        '        Print #pathFileNo, "# " & .path(j).pP(k).pOr.x & " " & .path(j).pP(k).pOr.y & " " & .path(j).pP(k).pOr.z
        '        Print #pathFileNo, "# flag0    of this point = " & .path(j).pP(k).flag0
        '        Print #pathFileNo, "# flag1    of this point = " & .path(j).pP(k).flag1
        '        Print #pathFileNo, "# flag2    of this point = " & .path(j).pP(k).flag2
        '        Print #pathFileNo, "# flag3    of this point = " & .path(j).pP(k).flag3
        '        For l = 1 To (.path(j).noPars - 4)
        '            Print #pathFileNo, "# flag4(" & l & ") of this point = " & .path(j).pP(k).flag4(l)
        '        Next l
        '    Next k
        '    Print #pathFileNo, "# " & .path(j).noPpoints & " vertices"
        '    For k = 1 To .path(j).noPpoints
        '        If k < .path(j).noPpoints Then
        '            Print #pathFileNo, "f  " & k & " " & (k + 1) & " " & (k + 1)
        '        End If
        '    Next k
        '    Print #pathFileNo, "# " & (.path(j).noPpoints - 1) & " faces"
        '    Print #pathFileNo, ""
        '    Close #pathFileNo
        'Next j
        ' End of output section.
        
    End With

End Sub

Private Sub readMrkr(openHODno As Integer, noMrkr As Integer, lFm As Long)

Dim mrkrFileNo As Integer

    ReDim Preserve myHOD.dtrmFb.mrkrCh(1 To noMrkr)
    
    With myHOD.dtrmFb.mrkrCh(noMrkr)
    
        .mrkrFh.tF = "FORM"
        .mrkrFh.lF = lFm
        .mrkrFb.tID = "MRKR"
        Get openHODno, , .mrkrFb.headFh.tF
        Get openHODno, , .mrkrFb.headFh.lF
        Get openHODno, , .mrkrFb.headFb.tID
        If .mrkrFb.headFb.tID <> "HEAD" Then
            MsgBox "No HEAD chunk? " & Seek(openHODno)
            Exit Sub
        End If
        Get openHODno, , .mrkrFb.headFb.noHds
        .mrkrFb.headFb.noHds = ReverseLong(.mrkrFb.headFb.noHds)
        If .mrkrFb.headFb.noHds <> 1 Then MsgBox "Number of markers is not equal to one: " & .mrkrFb.headFb.noHds & " (file pos. = " & Seek(openHODno) & ")."
        Get openHODno, , .mrkrFb.headFb.lN
        .mrkrFb.headFb.nHe = Space(.mrkrFb.headFb.lN)
        Get openHODno, , .mrkrFb.headFb.nHe
        Get openHODno, , .mrkrFb.headFb.lNpo
        .mrkrFb.headFb.nPo = Space(.mrkrFb.headFb.lNpo)
        Get openHODno, , .mrkrFb.headFb.nPo
        Get openHODno, , .mrkrFb.headFb.sepr
        Get openHODno, , .mrkrFb.headFb.position
        Get openHODno, , .mrkrFb.headFb.rotation
        Get openHODno, , .mrkrFb.headFb.keyfFh
        Get openHODno, , .mrkrFb.headFb.keyfFb.tID
        If .mrkrFb.headFb.keyfFb.tID <> "KEYF" Then
            MsgBox "No KEYF chunk? " & Seek(openHODno)
            Exit Sub
        End If
        
        ' Output in order to check points in drawing program.
        'mrkrFileNo = FreeFile
        'Open "allMarkers.obj" For Append As #mrkrFileNo
        'Print #mrkrFileNo, "g " & .mrkrFb.headFb.nHe
        'Print #mrkrFileNo, "v " & CSng(.mrkrFb.headFb.position.x) & " " & CSng(.mrkrFb.headFb.position.y) & " " & CSng(.mrkrFb.headFb.position.z)
        'Close #mrkrFileNo
        ' End of output section.
                
    End With
    
End Sub

Private Sub readCold(openHODno As Integer, noCold As Integer, lFc As Long)

Dim trisFileNo As Integer
Dim j As Long

    ReDim Preserve myHOD.dtrmFb.coldCh(1 To noCold)

    With myHOD.dtrmFb.coldCh(noCold).coldFh
        .tF = "FORM"
        .lF = .lF
    End With
    
    With myHOD.dtrmFb.coldCh(noCold).coldFb
        .tID = "COLD"
        Get openHODno, , .lO
        .nO = Space(.lO)
        Get openHODno, , .nO
        Get openHODno, , .bboxFb
        If .bboxFb.tID <> "BBOX" Then
            MsgBox "No BBOX chunk in COLD? " & .bboxFb.tID & " (file pos. " & Seek(openHODno) & ")."
            Exit Sub
        End If
        Get openHODno, , .bsphFb
        If .bsphFb.tID <> "BSPH" Then
            MsgBox "No BSPH chunk in COLD? " & .bsphFb.tID & " (file pos. " & Seek(openHODno) & ")."
            Exit Sub
        End If
        Get openHODno, , .trisFb.tID
        If .trisFb.tID <> "TRIS" Then
            MsgBox "No TRIS chunk in COLD? " & .trisFb.tID & " (file pos. " & Seek(openHODno) & ")."
            Exit Sub
        End If
        Get openHODno, , .trisFb.lF
        Get openHODno, , .trisFb.noVtcs
        ReDim .trisFb.vtx(1 To .trisFb.noVtcs)
        Get openHODno, , .trisFb.vtx
        Get openHODno, , .trisFb.noFcs
        .trisFb.noFcs = .trisFb.noFcs / 3
        ReDim .trisFb.fc(1 To .trisFb.noFcs)
        Get openHODno, , .trisFb.fc
        
        '  Output in order to check points in drawing program.
        'trisFileNo = FreeFile
        'Open "tris" & noCold & ".obj" For Append As #trisFileNo
        'Print #trisFileNo, ""
        'Print #trisFileNo, "g triangles"
        'For j = 1 To .trisFb.noVtcs
        '    Print #trisFileNo, "v " & .trisFb.vtx(j).x & " " & .trisFb.vtx(j).y & " " & .trisFb.vtx(j).z
        'Next j
        'Print #trisFileNo, "# " & .trisFb.noVtcs & " vertices"
        'For j = 1 To .trisFb.noFcs
        '    Print #trisFileNo, "f  " & (.trisFb.fc(j).vtx1 + 1) & " " & (.trisFb.fc(j).vtx2 + 1) & " " & (.trisFb.fc(j).vtx3 + 1)
        'Next j
        'Print #trisFileNo, "# " & .trisFb.noFcs & " faces"
        'Print #trisFileNo, ""
        'Close #trisFileNo
        ' End of output section.
        
    End With
    
End Sub

Private Sub readBsrm(openHODno As Integer, noBsrm As Integer, lFb As Long)

Dim j As Long
Dim k As Long
Dim trisFileNo As Integer
Dim ssubFileNo As Integer
Dim idxBlockChk As Double

    ReDim Preserve myHOD.dtrmFb.bsrmCh(1 To noBsrm)

    With myHOD.dtrmFb.bsrmCh(noBsrm).bsrmFh
        .tF = "FORM"
        .lF = lFb
    End With
    
    With myHOD.dtrmFb.bsrmCh(noBsrm).bsrmFb
        .tID = "BSRM"
        Get openHODno, , .descFh
        Get openHODno, , .descFb.tID
        If .descFb.tID <> "DESC" Then
            MsgBox "No DESC chunk in BSRM? (" & .descFh.tF & " " & .descFb.tID & ", file pos. " & Seek(openHODno) & ")."
            Exit Sub
        End If
        Get openHODno, , .descFb.altLf
        Get openHODno, , .descFb.lN
        .descFb.nM = Space(.descFb.lN)
        Get openHODno, , .descFb.nM
        Get openHODno, , .descFb.lO
        .descFb.nO = Space(.descFb.lO)
        Get openHODno, , .descFb.nO
        Get openHODno, , .descFb.nonGoblin
        Get openHODno, , .bboxFh
        Get openHODno, , .bboxFb
        If .bboxFb.tID <> "BBOX" Then
            MsgBox "No BBOX chunk in BSRM? (" & .bboxFh.tF & " " & .bboxFb.tID & ", file pos. " & Seek(openHODno) & ")."
            Exit Sub
        End If
        Get openHODno, , .bsphFh
        Get openHODno, , .bsphFb
        If .bsphFb.tID <> "BSPH" Then
            MsgBox "No BSPH chunk in BSRM? (" & .bsphFh.tF & " " & .bsphFb.tID & ", file pos. " & Seek(openHODno) & ")."
            Exit Sub
        End If
        Get openHODno, , .trisFh
        Get openHODno, , .trisFb.tID
        If .trisFb.tID <> "TRIS" Then
            MsgBox "No TRIS chunk in BSRM? (" & .trisFh.tF & " " & .trisFb.tID & ", file pos. " & Seek(openHODno) & ")."
            Exit Sub
        End If
        Get openHODno, , .trisFb.lF
        Get openHODno, , .trisFb.noVtcs
        ReDim .trisFb.vtx(1 To .trisFb.noVtcs)
        Get openHODno, , .trisFb.vtx
        Get openHODno, , .trisFb.noFcs
        .trisFb.noFcs = .trisFb.noFcs / 3
        ReDim .trisFb.fc(1 To .trisFb.noFcs)
        Get openHODno, , .trisFb.fc
        Get openHODno, , .vnrmFh
        Get openHODno, , .vnrmFb.tID
        If .vnrmFb.tID <> "VNRM" Then
            MsgBox "No VNRM chunk in BSRM? (" & .vnrmFh.tF & " " & .vnrmFb.tID & ", file pos. " & Seek(openHODno) & ")."
            Exit Sub
        End If
        Get openHODno, , .vnrmFb.altLf
        Get openHODno, , .vnrmFb.noNrmls
        ReDim .vnrmFb.nrml(1 To .vnrmFb.noNrmls)
        Get openHODno, , .vnrmFb.nrml
        Get openHODno, , .vnrmFb.noFcs
        .vnrmFb.noFcs = .vnrmFb.noFcs / 3
        If .vnrmFb.noFcs <> .trisFb.noFcs Then MsgBox "The number of faces in the TRIS & VRML chunks does not match. TRIS: " & .trisFb.noFcs & " VNRM: " & .vnrmFb.noFcs
        ReDim .vnrmFb.fc(1 To .vnrmFb.noFcs)
        Get openHODno, , .vnrmFb.fc
        Get openHODno, , .ssubFh
        Get openHODno, , .ssubFb.tID
        If .ssubFb.tID <> "SSUB" Then
            MsgBox "No SSUB chunk in BSRM? (" & .ssubFh.tF & " " & .ssubFb.tID & ", file pos. " & Seek(openHODno) & ")."
            Exit Sub
        End If
        Get openHODno, , .ssubFb.altLf
'***
        Get openHODno, , .ssubFb.flag
'***
        .ssubFh.lF = ((ReverseLong(.ssubFh.lF) - 24) / 2)
        ReDim .ssubFb.fcIdx(1 To .ssubFh.lF)
        Get openHODno, , .ssubFb.fcIdx
        
        ' Output in order to validate DESC & SSUB parameters.
        trisFileNo = FreeFile
        Open "bmshtris" & noBsrm & ".obj" For Append As trisFileNo
        Print #trisFileNo, ""
        Print #trisFileNo, "g " & .descFb.nM
        For j = 1 To .trisFb.noVtcs
            Print #trisFileNo, "v " & .trisFb.vtx(j).X & " " & .trisFb.vtx(j).Y & " " & .trisFb.vtx(j).Z
        Next j
        Print #trisFileNo, "# " & .trisFb.noVtcs & "vertices"
        For j = 1 To .trisFb.noFcs
            Print #trisFileNo, "f  " & (.trisFb.fc(j).vtx1 + 1) & " " & (.trisFb.fc(j).vtx2 + 1) & " " & (.trisFb.fc(j).vtx3 + 1)
        Next j
        Print #trisFileNo, "# " & .trisFb.noFcs & " faces"
        Print #trisFileNo, ""
        Close #trisFileNo
        ssubFileNo = FreeFile
        Open "ssubCheck" & noBsrm & ".txt" For Append As #ssubFileNo
        For j = 1 To .ssubFh.lF Step 6
            If (j + 5) <= .ssubFh.lF Then
                Print #ssubFileNo, (.ssubFb.fcIdx(j) + 1) & " " & (.ssubFb.fcIdx(j + 1) + 1) & " " & (.ssubFb.fcIdx(j + 2) + 1) & " " _
                                    & (.ssubFb.fcIdx(j + 3) + 1) & " " & (.ssubFb.fcIdx(j + 4) + 1) & " " & (.ssubFb.fcIdx(j + 5) + 1)
                                    ' Plus one is convenient for Excel & OBJ-file face comparison...
            Else
                For k = j To .ssubFh.lF
                    Print #ssubFileNo, (.ssubFb.fcIdx(k) + 1)
                Next k
            End If
        Next j
        Print #ssubFileNo, ""
        Close #ssubFileNo
        ' End of output section.
        
    End With
        
End Sub

Private Sub readBurn(openHODno As Integer, noBurn As Integer)

Dim j As Integer
Dim burnFileNo As Integer

    ReDim Preserve myHOD.dtrmFb.burnFb(1 To noBurn)

    With myHOD.dtrmFb.burnFb(noBurn)
    
        .tID = "BURN"
        Get openHODno, , .lF
        Get openHODno, , .lN
        .nB = Space(.lN)
        Get openHODno, , .nB
        Get openHODno, , .lHo
        .nHo = Space(.lHo)
        Get openHODno, , .nHo
        Get openHODno, , .noConnPnts
        Get openHODno, , .unknown4
        If .unknown4 <> 1 Then MsgBox "With BURN" & noBurn & " 'unknown4' is unequal to one: " & .unknown4 & " (at file pos. " & Seek(openHODno) & ")."
        ReDim .cPnt(1 To .noConnPnts)
        Get openHODno, , .cPnt
        
        ' Output in order to check points in drawing program.
        'burnFileNo = FreeFile
        'Open "burn" & noBurn & "Cpoints.obj" For Append As #burnFileNo
        'Print #burnFileNo, ""
        'Print #burnFileNo, "g " & .nB
        'For j = 1 To .noConnPnts
        '    Print #burnFileNo, "v " & .cPnt(j).x & " " & .cPnt(j).y & " " & .cPnt(j).z
        'Next j
        'Print #burnFileNo, "# " & .noConnPnts & " vertices"
        'Print #burnFileNo, ""
        'Close #burnFileNo
        ' End of output section.
        
    End With

End Sub

Private Sub readBndv(openHODno As Integer)
    
Dim j As Integer
Dim bndvFileNo As Integer

    With myHOD.dtrmFb.bndvFb
    
        .tID = "BNVD"
        Get openHODno, , .lF
        Get openHODno, , .lN
        .nAll = Space(.lN)
        Get openHODno, , .nAll
        Get openHODno, , .lO
        .nO = Space(.lO)
        Get openHODno, , .nO
        Get openHODno, , .noGenPts
        If .noGenPts <> 8 Then MsgBox "The BNDV parameter 'noGenPts' is unequal to eight: " & .noGenPts & " (file pos. " & Seek(openHODno) & ")."
        Get openHODno, , .leftLowerCorner
        Get openHODno, , .sepr1
        For j = 1 To 15
            If .sepr1(j) <> 0 Then MsgBox "The BNDV param. 'sepr1(" & j & ") is not equal to zero (file pos. " & Seek(openHODno) & ")."
        Next j
        Get openHODno, , .rightUpperCorner
        Get openHODno, , .sepr2
        For j = 1 To 4
            If .sepr2(j) <> 0 Then MsgBox "The BNDV param. 'sepr2(" & j & ") is not equal to zero (file pos. " & Seek(openHODno) & ")."
        Next j
        
        ' Output in order to check points in drawing program.
        If .noGenPts <> 8 Then
            bndvFileNo = FreeFile
            Open "bndvBpoints.obj" For Append As #bndvFileNo
            Print #bndvFileNo, ""
            Print #bndvFileNo, "g " & .nO & "_" & .nAll
            Print #bndvFileNo, "v " & .leftLowerCorner.X & " " & .leftLowerCorner.Y & " " & .leftLowerCorner.Z
            Print #bndvFileNo, "v " & .rightUpperCorner.X & " " & .rightUpperCorner.Y & " " & .rightUpperCorner.Z
            Print #bndvFileNo, "# 2 vertices"
            Print #bndvFileNo, ""
            Close #bndvFileNo
        End If
        ' End of output section.
        
    End With
    
End Sub

Private Sub readAnim(openHODno As Integer, noAnim As Integer, lFb As Long)

Dim j As Integer
Dim k As Integer
Dim animFileNo As Integer

    ReDim Preserve myHOD.dtrmFb.animCh(1 To noAnim)

    With myHOD.dtrmFb.animCh(noAnim).animFh
        .tF = "NRML"
        .lF = lFb
    End With
    
    With myHOD.dtrmFb.animCh(noAnim).animFb
        .tID = "ANIM"
        Get openHODno, , .noAnimPars
        If ReverseLong(.noAnimPars) <> 1 Then MsgBox "The ANIM parameter noAnimPars is unequal to one: " & .noAnimPars & " (file pos. " & Seek(openHODno) & ")."
        Get openHODno, , .lNpar
        .nPar = Space(.lNpar)
        Get openHODno, , .nPar
        Get openHODno, , .noKeyfs
        ReDim .keyframe(1 To .noKeyfs)
        Get openHODno, , .keyframe
        Get openHODno, , .sepr(1)
        Get openHODno, , .sepr(2)
        
        ' Output keyframe parameters in order to check & compare.
        animFileNo = FreeFile
        Open "animCheck.txt" For Append As #animFileNo
        Print #animFileNo, "Keyframes for " & .nPar
        Print #animFileNo, ""
        For j = 1 To .noKeyfs
            Print #animFileNo, "keyframe " & j
            Print #animFileNo, "time value: " & CSng(.keyframe(j).kTime)
            Print #animFileNo, "pos. / rot. : " & CSng(.keyframe(j).kPos.X) & " " & CSng(.keyframe(j).kPos.Y) & " " & CSng(.keyframe(j).kPos.Z)
            Print #animFileNo, "pos. / rot. : " & CSng(.keyframe(j).kRot.X) & " " & CSng(.keyframe(j).kRot.Y) & " " & CSng(.keyframe(j).kRot.Z)
            Print #animFileNo, ""
        Next j
        Print #animFileNo, "sepr1: " & .sepr(1)
        Print #animFileNo, "sepr2: " & .sepr(2)
        Print #animFileNo, ""
        Print #animFileNo, ""
        Close #animFileNo
        ' End of output section
        
    End With

End Sub

Public Function exportDDS(fileName As String, dxt5 As dxt5B) As String

' Write out the DDS file:
' a standard DDS header will have
' DDS File constants:
Const DDS_DWMAGIC As String = "DDS "
Const DDS_DWSIZE As Long = 124
Const DDS_DWFLAGS As Long = 659463
Const DDS_DWDEPTH As Long = 0
Const DDS_PIXELFORMAT_DWSIZE As Long = 32
Const DDS_PIXELFORMAT_DWFLAGS As Long = 4
Const DDS_PIXELFORMAT_DWRGBBITCOUNT As Long = 0
Const DDS_PIXELFORMAT_DWRBITMASK As Long = 0
Const DDS_PIXELFORMAT_DWGBITMASK As Long = 0
Const DDS_PIXELFORMAT_DWBBITMASK As Long = 0
Const DDS_PIXELFORMAT_DWRGBALPHABITMASK As Long = 0
Const DDS_DDSCAPS_DWCAPS1 As Long = 4198408
Const DDS_DDSCAPS_DWCAPS2 As Long = 0
Const DDS_DDSCAPS_DWRESERVED As Long = 0
Const DDS_DWRESERVED As Long = 0

Dim DDSfileOK As Boolean
Dim i As Integer
Dim j As Integer
Dim k As Integer
Dim myDDS As Integer

    With dxt5
      myDDS = FreeFile
      exportDDS = fileName
      Open (exportDDS) For Binary As myDDS
      Put myDDS, , DDS_DWMAGIC
      Put myDDS, , DDS_DWSIZE
      Put myDDS, , DDS_DWFLAGS
      Put myDDS, , .Hi
      Put myDDS, , .Wi
      Put myDDS, , .Hi * .Wi
      Put myDDS, , DDS_DWDEPTH
      Put myDDS, , .noMIPls
      For k = 1 To 11
          Put myDDS, , DDS_DWRESERVED
      Next k
      Put myDDS, , DDS_PIXELFORMAT_DWSIZE
      Put myDDS, , DDS_PIXELFORMAT_DWFLAGS
      Put myDDS, , .tID
      Put myDDS, , DDS_PIXELFORMAT_DWRGBBITCOUNT
      Put myDDS, , DDS_PIXELFORMAT_DWRBITMASK
      Put myDDS, , DDS_PIXELFORMAT_DWGBITMASK
      Put myDDS, , DDS_PIXELFORMAT_DWBBITMASK
      Put myDDS, , DDS_PIXELFORMAT_DWRGBALPHABITMASK
      Put myDDS, , DDS_DDSCAPS_DWCAPS1
      Put myDDS, , DDS_DDSCAPS_DWCAPS2
      Put myDDS, , DDS_DDSCAPS_DWRESERVED
      Put myDDS, , DDS_DDSCAPS_DWRESERVED
      Put myDDS, , DDS_DWRESERVED
      Put myDDS, , .Body
      Close myDDS
    End With
    
End Function
