VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCore"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Public Enum cfeModality
   cfeModal = vbModal
   cfeModeless = vbModeless
End Enum

Public errModule As String
Public errFunction As String
Public errParms As String

Type pluginInfoType
  name As String
  Author As String
  version As String
  pluginType As pluginType
  fileExtension As String
  fileDescription As String
  notes As String
End Type

Type hwsePLUGINtype
    PluginInfo As pluginInfoType
    strClassName As String
End Type

Private nPlugins() As hwsePLUGINtype

'Private nPlugins() As clsPluginType
Public numPlugins As Integer

Public Sub ShowError(Optional modality As _
      cfeModality = cfeModal)
   Dim frm As New frmError
   'frm.ShowErrorForm errModule, errorFunction, errorParms, modality
   frm.errModule = errModule
   frm.errFunction = errFunction
   frm.errParms = errParms
   frm.Show modality
End Sub

Public Sub ShowAbout(Optional modality As _
      cfeModality = cfeModal)
   Dim frm As New frmAbout
   frm.Show modality
End Sub

Public Function checkForBackup(filename As String) As Boolean
  checkForBackup = checkForBackup2(filename)
End Function

Public Property Get GetPluginInfo(id As Long) As hwsePLUGINtype
  GetPluginInfo = nPlugins(id)
End Property

Public Function LoadPlugins(pluginsDir As String, myobjHost As IHostInterface) As Boolean
    Dim strDlls() As String
    Dim iCounter As Integer, i As Integer, j As Integer
    Dim iPluginCount As Integer, iFailCount As Integer
    Dim strInterfaceName As String
    
    Dim objTypeInfo As TypeLibInfo

    'Initialize dynamic array of plugin information
    hwseLog "Initialising plugin array..."
    ReDim nPlugins(0)
    
    'Set up our interface which we'll pass to plugins to they can call back
    'Set objHost = New clsHostInterface
    
    'Enumerate DLL files in plugins directory
    If EnumFiles(pluginsDir & "*.dll", strDlls) = True Then
        For iCounter = 0 To UBound(strDlls)
            'Get typelib information from the DLL file
            hwseLog "  " & iCounter & pluginsDir & strDlls(iCounter)
            
            Set objTypeInfo = TLI.TypeLibInfoFromFile(pluginsDir & strDlls(iCounter))
            hwseLog "  Attempting to register..."
            objTypeInfo.Register
      
            For j = 1 To objTypeInfo.TypeInfos.Count
                For i = 1 To objTypeInfo.TypeInfos(j).Interfaces.Count
                    'This is where we'll get an error if something is wrong with the DLL itself
                    On Local Error Resume Next
                    strInterfaceName = objTypeInfo.TypeInfos(j).Interfaces(i).name
                    hwseLog "Error: " & Err
                    If Err Then
                        hwseLog strInterfaceName & " Failed to load plugin " & Chr$(34) & strDlls(iCounter) & Chr$(34)
                        myobjHost.DisplayText "Failed to load plugin " & Chr$(34) & strDlls(iCounter) & Chr$(34)
                        strInterfaceName = ""
                        iFailCount = iFailCount + 1
                        Err.Clear
                    End If
                    On Error GoTo 0
              
                    hwseLog "interface name: " & strInterfaceName
                    'Test for required interfaces
                    If Mid$(strInterfaceName, 1, 5) = "_hwse" Then
                        'We've found what appears to be a compatible plugin
                        If LoadPlugin(pluginsDir & strDlls(iCounter), objTypeInfo.name & "." & objTypeInfo.TypeInfos(j).name) = False Then
                            myobjHost.DisplayText "Failed to load plugin " & Chr$(34) & strDlls(iCounter) & Chr$(34)
                        Else
                            myobjHost.DisplayText "Loaded plugin: " & strDlls(iCounter) & " v" & nPlugins(UBound(nPlugins)).PluginInfo.version
                            'myobjHost.DisplayText "--> " & nPlugins(UBound(nPlugins)).PluginInfo.pluginType
                            iPluginCount = iPluginCount + 1
                        End If
                    End If
                Next i
            Next j
        Next
    Else
        LoadPlugins = False
        Exit Function
    End If
    
    'See if we loaded any plugins successfully
    If iPluginCount > 0 Then
        myobjHost.DisplayText CStr(iPluginCount) & " Plugin DLL(s) loaded successfully."
        numPlugins = iPluginCount
        LoadPlugins = True
    Else
        numPlugins = 0
        LoadPlugins = False
    End If
End Function

Private Function LoadPlugin(strDLLPath As String, strClassName As String) As Boolean

    hwseLog "LoadPlugin: "
    hwseLog "--> " & strDLLPath
    hwseLog "--> " & strClassName

    Dim objPlugin As hwsePlugin
    Dim bError As Boolean

    On Local Error Resume Next

    'Try and create an instance of the plugin
    Set objPlugin = CreateObject(strClassName)
    hwseLog "Error: " & Err

    'If we fail, try registering the DLL
    If Err Then
        'This is probably because the DLL isn't registered, so we try and register it
        If RegisterServer(0, strDLLPath, True) = True Then
            Err.Clear
            Set objPlugin = CreateObject(strClassName)
            If Err Then bError = True
        Else
            bError = True
        End If
    End If

    hwseLog " " & bError & " - " & strDLLPath

    'bError indicates whether we've been able to create an instance of the plugin
    If bError = True Then
        LoadPlugin = False
    Else
        'NOTE: We don't actually store an instance of the plugin in our array.
        'This would probably be the best way to do it for this tiny sample
        'application, but in real life it is usually best to create and destroy
        'instances when we need them. We do, however keep a copy of the plugin
        'information.

        'Create a new entry in our plugin database for this plugin
        ReDim Preserve nPlugins(UBound(nPlugins) + 1)
        'nPlugins(UBound(nPlugins)) = objPlugin
        nPlugins(UBound(nPlugins)).strClassName = strClassName
        nPlugins(UBound(nPlugins)).PluginInfo = objPlugin.PluginInfo
        nPlugins(UBound(nPlugins)).PluginInfo.version = GetVersionInfo(strDLLPath)
        LoadPlugin = True
    End If

    hwseLog "LoadPlugin result: " & LoadPlugin

    Set objPlugin = Nothing
End Function

Public Sub hwseLog(message As String)
  
  Dim appLogFile As Integer
  Dim appLogFilename As String
  
  appLogFile = FreeFile
  appLogFilename = App.Path & "\hwse.log"
  
  Open appLogFilename For Append As #appLogFile
  Print #appLogFile, message
  Close #appLogFile

End Sub

Public Function getPluginForFileType(extension As String, objHost As IHostInterface) As hwsePlugin
  ' Iterate through plugins to find if there
  ' is one for this texture type
  Dim Count As Long, fileExt As String, i As Integer, checkFileExt As String
  For Count = 1 To numPlugins
    fileExt = GetPluginInfo(Count).PluginInfo.fileExtension
    For i = 1 To countTokens(fileExt, ";")
      checkFileExt = Replace(getWord(fileExt, i, Asc(";")), "*", "")
      'Debug.Print i & ": " & checkFileExt
    
      If LCase(checkFileExt) = LCase(extension) Then
        ' We found the plugin that deals with this file extension, so we
        ' have to create an instance of this plugin
        Set getPluginForFileType = CreateObject(GetPluginInfo(Count).strClassName)
        If getPluginForFileType.InitPlugin(objHost) = False Then Exit Function
      End If
    Next
  Next Count

End Function

Private Sub Class_Initialize()
  If Dir(App.Path & "\hwse.log") <> "" Then
    Kill App.Path & "\hwse.log"
  End If
End Sub

